/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window with a grid populated with messages for the specified creator.
 */
Ext.define( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.ViewError',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.scheduler.common.message.MessageQueueWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /*
   * Required ID of the messages creator.
   */
  creatorId : undefined,

  /*
   * Required parameter with the message creator type ({@link QDesk.model.message.MessageCreatorType}).
   */
  creatorType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.creatorId )
      Ext.Error.raise( 'Required config parameter missing: creatorId' );

    if ( !config.creatorType )
      Ext.Error.raise( 'Required config parameter missing: creatorType' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'MessageQueueWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx
      }
    } );

    me._bindEventHandlers( window );

    var grid = window.down( '#messageQueueGrid' );
    var store = grid.getStore();

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          window.show();

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );
    } );

    var grid = window.down( '#messageQueueGrid' );
    var store = grid.getStore();

    // in before load we add all required parameters
    store.on( 'beforeload', function ( store, operation )
    {
      var params = operation.getParams() || {};

      if ( grid.filtersValid() )
      {
        store.getProxy().url = QDesk.util.constant.ActionUrl.messageActionListForCreator;

        Ext.apply( params, {
          connectionId : me.connectionCtx.getConnectionId(),
          creatorId : me.creatorId,
          creatorType : me.creatorType
        } );

        var fetchPeriod = grid.down( '#fetchPeriod' ).getValue();
        Ext.apply( params, {
          fetchPeriod : Ext.encode( fetchPeriod )
        } );

        var maxFetchSize = grid.down( '#maxFetchSizeSelect' ).getValue();
        if ( maxFetchSize != -1 )  // -1 == unlimited
        {
          Ext.apply( params, {
            maxFetchSize : maxFetchSize
          } );
        }
      }
      else
      {
        store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      }

      operation.setParams( params );

      grid.down( '#refreshButton' ).setDisabled( true );
      grid.down( '#fetchPeriod' ).setDisabled( true );
      grid.down( '#maxFetchSizeSelect' ).setDisabled( true );
    } );

    store.on( 'load', function ( store, operation )
    {
      grid.down( '#refreshButton' ).setDisabled( false );
      grid.down( '#fetchPeriod' ).setDisabled( false );
      grid.down( '#maxFetchSizeSelect' ).setDisabled( false );
    } );

    // opens a window showing the last error description of the specified message
    grid.down( '#lastErrorDescColumn' ).on( 'requestViewError', function ( lastErrorDesc )
    {
      Ext.create( 'QDesk.controller.command.ViewError', {
        maskEl : window.getEl(),
        errorText : lastErrorDesc
      } ).execute();
    } );

    grid.down( '#refreshButton' ).on( 'click', function ()
    {
      me._refreshGrid( window );
    } );

    grid.on( 'changeFilters', function ( combo, newValue, oldValue )
    {
      me._refreshGrid( window );
    } );
  },


  /**
   * Refreshes the contents of the message grid.
   * @param window the messages window.
   * @private
   */
  _refreshGrid : function ( window )
  {
    var me = this;

    var grid = window.down( '#messageQueueGrid' );
    var store = grid.getStore();

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        if ( !success )
        {
          store.removeAll( false );

          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  }
} );
