/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the view panel for the Quartz scheduler described by the specified
 * connection description configuration object.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.quartz.RefreshSchedulerView',

    'QDesk.controller.command.scheduler.common.StartScheduler',
    'QDesk.controller.command.scheduler.common.PauseScheduler',
    'QDesk.controller.command.scheduler.common.StopScheduler',

    'QDesk.controller.command.scheduler.quartz.audit.InitializeSchedulerAuditTrailTab',
    'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailTab',

    'QDesk.controller.command.scheduler.quartz.chain.RefreshJobChainsTab',

    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',

    'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsTab',

    'QDesk.controller.command.scheduler.quartz.misfire.InitializeSchedulerMisfiredTriggersTab',
    'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersTab',

    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshExecNotificationsTab',

    'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',

    'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',

    'QDesk.model.license.ProductEdition',

    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProduct'
  ],

  /**
   * Required QDesk.ConnectionContext instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView(
        me.connectionCtx.getConnectionId() );

    // if there is no tab panel for the scheduler, create a new one and add it to the mainTabPanel, otherwise
    // just activate the existing tab panel
    if ( schedulerView == null )
    {
      me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.schedulerActionAgentInfo,
        params : {
          connectionId : me.connectionCtx.getConnectionId()
        },
        callback : function ( opts, success, resp )
        {
          me.unmask();

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              /*
               * Update JVM agent attributes and add all missing JVM agent info attributes that were not,
               * for performance reasons, available when the ConnectionCtx instance was created from the
               * data extracted from the double-clicked ConnectionTreeNode.
               */
              me.connectionCtx.getJvmAgentInfo().update( json.data );

              me._doChecksAndOpenSchedulerView( mainTabPanel );
            }
            else
            {
              // Server side error
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
          else
          {
            QDesk.util.UxUtils.showExtJsAjaxError( resp );
          }
        }
      } );
    }
    else
    {
      // make the panel active
      mainTabPanel.setActiveTab( schedulerView );
    }
  },


  _doChecksAndOpenSchedulerView : function ( mainTabPanel )
  {
    var me = this;

    var jvmAgentInfo = me.connectionCtx.getJvmAgentInfo();

    if ( jvmAgentInfo.isDetected() )
    {
      if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.model.license.ProductEdition.LITE )
      {
        // For QuartzDesk Lite edition we SKIP QuartzDesk JVM Agent version check!
        me._openSchedulerView( mainTabPanel );
      }
      else
      {
        // For QuartzDesk Standard and Enterprise editions we PERFORM QuartzDesk JVM Agent version and edition check!

        /* compatible version and edition */
        if ( jvmAgentInfo.isCompatible() )
        {
          // *** agent version and edition are compatible ***
          me._checkPublicApi( function ()
          {
            me._openSchedulerView( mainTabPanel );
          } );
        }
        else
        {
          if ( jvmAgentInfo.isCompatibleVersion() )
          {
            if ( jvmAgentInfo.isCompatibleVersion() )
            {
              // *** agent edition and version are compatible ***
              me._checkPublicApi( function ()
              {
                me._openSchedulerView( mainTabPanel );
              } );
            }
            else
            {
              // *** agent edition is not compatible ***
              var productEdition = QDesk.util.license.LicensedProduct.productEdition;

              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentIncompatibleEditionMsg',
                      jvmAgentInfo.getEdition(),
                      productEdition
                  ),
                  function ()
                  {
                    me._openSchedulerView( mainTabPanel );
                  } );
            }
          }
          else
          {
            // *** agent version is not compatible ***
            var productVersion = QDesk.util.license.LicensedProduct.productVersion;

            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentIncompatibleVersionMsg',
                    QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ),
                    QDesk.util.FormatUtils.formatVersion( productVersion )
                ),
                function ()
                {
                  me._openSchedulerView( mainTabPanel );
                } );
          }
        }
        /* End of QuartzDesk JVM Agent vs QuartzDesk app version check */
      }
    }
    else
    {
      //
      // JVM agent has not been detected
      //

      // Lite edition
      if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.model.license.ProductEdition.LITE )
      {
        // For Lite editions we IGNORE that the QuartzDesk JVM Agent has not been detected!
        me._openSchedulerView( mainTabPanel );
      }
      else
      {
        // Standard and Enterprise editions
        if ( QDesk.ApplicationConfig.get( 'warning.jvmAgentNotDetected' ) )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentNotDetectedMsg' ),
              function ()
              {
                me._openSchedulerView( mainTabPanel );
              } );
        }
        else
        {
          me._openSchedulerView( mainTabPanel );
        }
      }
    }
  },

  /**
   * Checks whether the QuartzDesk Public API Library has been detected, loaded correctly and has a version that is
   * compatible with the installed QuartzDesk JVM Agent. When done, this method executes the specified callback
   * function.
   * Please note that this function is called iff the agent has been detected and is compatible.
   *
   * @param callback a callback function.
   * @private
   */
  _checkPublicApi : function ( callback )
  {
    var me = this;

    var jvmAgentInfo = me.connectionCtx.getJvmAgentInfo();
    var schedulerInfo = me.connectionCtx.getSchedulerInfo();

    if ( schedulerInfo.isPublicApiReady() )
    {
      // *** public API has been detected, is loaded correctly and it has a compatible version
      callback.apply( this );
    }
    else
    {
      if ( !schedulerInfo.isPublicApiDetected() )
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.publicApiNotDetectedMsg'
            ),
            callback );
      }
      else if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.publicApiNotLoadedCorrectlyMsg'
            ),
            callback );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.publicApiIncompatibleVersionMsg',
                QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
                QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() )
            ),
            callback );
      }
      else
      {
        Ext.Error.raise( 'Unexpected QuartzDesk Public API Library configuration.' );
      }
    }
  },


  /**
   * Opens the Quartz scheduler view and adds it to the specified main tab panel.
   * @param mainTabPanel the main tab panel.
   */
  _openSchedulerView : function ( mainTabPanel )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

    // deferred so that the mask message gets displayed
    Ext.defer( function ()
    {
      var me = this;

      var connectionId = me.connectionCtx.getConnectionId();

      var schedulerView = mainTabPanel.add( {
        xtype : 'QuartzSchedulerView',

        itemId : QDesk.controller.command.connection.ConnectionUtils.getSchedulerViewItemId( connectionId ),

        title : Ext.String.htmlEncode(
            QDesk.controller.command.connection.ConnectionUtils.getConnectionPanelTitle( connectionId ) ),

        qdesk : {
          connectionCtx : me.connectionCtx
        }
      } );

      mainTabPanel.setActiveTab( schedulerView );

      me._initializeSchedulerView( schedulerView );

      me.unmask();
    }, 100, me );
  },


  /**
   * Initializes the scheduler view (panel) by binding all UI panel controls with the logic.
   * @param schedulerView a Quartz scheduler view.
   */
  _initializeSchedulerView : function ( schedulerView )
  {
    var me = this;

    var connectionId = me.connectionCtx.getConnectionId();

    var refreshButton = schedulerView.down( '#schedulerViewToolBar > #refreshButton' );
    refreshButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshSchedulerView', {
        connectionCtx : me.connectionCtx
      } ).execute();
    } );

    var startButton = schedulerView.down( '#schedulerViewToolBar > #startSchedulerButton' );
    startButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.StartScheduler', {
        connectionId : connectionId
      } ).execute();
    } );

    var pauseButton = schedulerView.down( '#schedulerViewToolBar > #pauseSchedulerButton' );
    pauseButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.PauseScheduler', {
        connectionId : connectionId
      } ).execute();
    } );

    var stopButton = schedulerView.down( '#schedulerViewToolBar > #stopSchedulerButton' );
    stopButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.StopScheduler', {
        connectionId : connectionId
      } ).execute();
    } );

    me._initializeSubTabs( schedulerView );
  },


  /**
   * Initializes sub tabs of the specified scheduler view.
   * @param schedulerView a scheduler view.
   */
  _initializeSubTabs : function ( schedulerView )
  {
    var me = this;

    var refreshButton = schedulerView.down( '#schedulerViewToolBar > #refreshButton' );

    var jobsTab = schedulerView.down( '#jobsTab' );
    jobsTab.on( 'activate', function ( jobsTab )
    {
      /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !jobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var executingJobsTab = schedulerView.down( '#executingJobsTab' );
    executingJobsTab.on( 'activate', function ( executingJobsTab )
    {
      /* If the executing jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !executingJobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var plannedJobExecutionsTab = schedulerView.down( '#plannedJobExecutionsTab' );
    plannedJobExecutionsTab.on( 'activate', function ( plannedJobExecutionsTab )
    {
      /* If the planned job executions tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !plannedJobExecutionsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // disable the Refresh button in the lower toolbar
      refreshButton.setDisabled( true );
    } );


    var triggersTab = schedulerView.down( '#triggersTab' );
    triggersTab.on( 'activate', function ( triggersTab )
    {
      /* If the triggers tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !triggersTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var misfiredTriggersTab = schedulerView.down( '#misfiredTriggersTab' );
    misfiredTriggersTab.on( 'activate', function ( misfiredTriggersTab )
    {
      /* If the misfired triggers tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !misfiredTriggersTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.misfire.InitializeSchedulerMisfiredTriggersTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var execHistoryTab = schedulerView.down( '#execHistoryTab' );
    execHistoryTab.on( 'activate', function ( execHistoryTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !execHistoryTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab', {
          connectionCtx : me.connectionCtx
        } ).execute();


        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var statisticsTab = schedulerView.down( '#statisticsTab' );
    statisticsTab.on( 'activate', function ( statisticsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !statisticsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var execNotificationRulesTab = schedulerView.down( '#execNotificationRulesTab' );
    execNotificationRulesTab.on( 'activate', function ()
    {
      // Note: initialization stuff moved to the relevant controllers

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var jobChainsTab = schedulerView.down( '#jobChainsTab' );
    jobChainsTab.on( 'activate', function ( jobChainsTab )
    {
      // Note: initialization stuff moved to the relevant controllers

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var auditTrailTab = schedulerView.down( '#auditTrailTab' );
    auditTrailTab.on( 'activate', function ( auditTrailTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !auditTrailTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.audit.InitializeSchedulerAuditTrailTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var infoTab = schedulerView.down( '#infoTab' );
    infoTab.on( 'activate', function ( infoTab )
    {
      /* If the info tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !infoTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab', {
          connectionCtx : me.connectionCtx
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    /* Initially selected tab (this could be made configurable...) */
    schedulerView.setActiveTab( jobsTab );

    // setActiveTab does not fire the activate event if the tab is already active so we need to
    // simulate the active event...

    /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
    if ( !jobsTab.getInitialized() )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }

    Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshSchedulerView', {
      connectionCtx : me.connectionCtx
    } ).execute();
  }
} );
