/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the SchedulerView of the connection whose {@link ConnectionContext} is passed in the
 * {@code connectionCtx} config parameter.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.RefreshSchedulerView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailTab',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshJobChainsTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',
    'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsTab',
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',

    'QDesk.model.scheduler.SchedulerStatus',

    'QDesk.util.constant.ActionUrl'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView(
        me.connectionCtx.getConnectionId() );

    //assert schedulerView != null

    /*
     * Refreshes the scheduler view tool bar (depends on the scheduler status).
     */
    me._updateSchedulerViewToolBarStatus( schedulerView );

    var activeTab = schedulerView.getActiveTab();

    if ( activeTab.getItemId() == 'jobsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'executingJobsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'plannedJobExecutionsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'triggersTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'misfiredTriggersTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'execHistoryTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'statisticsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'execNotificationRulesTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshExecNotificationsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'jobChainsTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshJobChainsTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'auditTrailTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeTab.getItemId() == 'infoTab' )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Unrecognized scheduler view tab ID (itemId): ' + activeTab.getItemId() );
    }
  },


  /**
   * Fetches info data for the specified scheduler view and updates the scheduler view toolbar using the data.
   * @param schedulerView the scheduler view.
   */
  _updateSchedulerViewToolBarStatus : function ( schedulerView )
  {
    var me = this;

    /* Disable all toolbar buttons */
    var refreshButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #refreshButton',
        schedulerView )[ 0 ];
    refreshButton.disable();

    var startSchedulerButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #startSchedulerButton',
        schedulerView )[ 0 ];
    startSchedulerButton.disable();

    var pauseSchedulerButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #pauseSchedulerButton',
        schedulerView )[ 0 ];
    pauseSchedulerButton.disable();

    var stopSchedulerButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #stopSchedulerButton',
        schedulerView )[ 0 ];
    stopSchedulerButton.disable();

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerActionInfo,
      params : {
        connectionId : me.connectionCtx.getConnectionId()
      },
      success : function ( response )
      {
        var json = Ext.decode( response.responseText );
        if ( json.success )
        {
          me._updateSchedulerViewToolBarButtons( schedulerView, json.data.status );
        }
        else
        {
          QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.schedulerActionInfo +
              ' to obtain scheduler data.' );

          me._updateSchedulerViewToolBarButtons( schedulerView /* unknown scheduler status */ );
        }
      },
      failure : function ( response )
      {
        QDesk.Log.error( 'Error invoking ' + QDesk.util.constant.ActionUrl.schedulerActionInfo +
            ' to obtain scheduler data.' );

        me._updateSchedulerViewToolBarButtons( schedulerView /* unknown scheduler status */ );
      }
    } );
  },


  /**
   * Enables/disabled the scheduler view toolbar buttons based on the current scheduler status.
   * @param schedulerView a scheduler view (panel).
   * @param schedulerStatus the current scheduler status.
   */
  _updateSchedulerViewToolBarButtons : function ( schedulerView, schedulerStatus )
  {
    var me = this;

    var refreshButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #refreshButton', schedulerView )[ 0 ];

    // scheduler buttons
    var startSchedulerButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #startSchedulerButton',
        schedulerView )[ 0 ];

    var pauseSchedulerButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #pauseSchedulerButton',
        schedulerView )[ 0 ];

    var stopSchedulerButton = Ext.ComponentQuery.query( '#schedulerViewToolBar > #stopSchedulerButton',
        schedulerView )[ 0 ];

    // refresh button should always be enabled regardless of the scheduler state
    refreshButton.enable();

    if ( schedulerStatus )
    {
      switch ( schedulerStatus )
      {
        case QDesk.model.scheduler.SchedulerStatus.STARTED:
          startSchedulerButton.disable();
          pauseSchedulerButton.enable();
          stopSchedulerButton.enable();
          break;

        case QDesk.model.scheduler.SchedulerStatus.PAUSED:
          startSchedulerButton.enable();
          pauseSchedulerButton.disable();
          stopSchedulerButton.enable();
          break;

        case QDesk.model.scheduler.SchedulerStatus.STOPPED:
          // not necessary as the tab of a stopped Quartz scheduler gets closed
          startSchedulerButton.disable();
          pauseSchedulerButton.disable();
          stopSchedulerButton.disable();
          break;

        default:
          Ext.Error.raise( 'Unrecognized scheduler status: ' + schedulerStatus )
      }
    }
  }
} );
