/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Audit Trail tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.audit.InitializeSchedulerAuditTrailTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailGrid',

    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var auditTrailTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getAuditTrailTab(
        me.connectionCtx.getConnectionId() );

    me._bindEventHandlers( auditTrailTab );

    auditTrailTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Audit Trail tab to their respective handlers.
   */
  _bindEventHandlers : function ( auditTrailTab )
  {
    var me = this;

    var auditTrailGrid = auditTrailTab.down( '#auditTrailGrid' );
    me._bindEventHandlersAuditTrailGrid( auditTrailGrid );
  },


  /**
   * Binds the Job Chains grid events to their handlers.
   */
  _bindEventHandlersAuditTrailGrid : function ( auditTrailGrid )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;
    if ( licensedFeatures.AuditTrail )
    {
      // refresh button
      auditTrailGrid.on( 'refresh',
          function ()
          {
            me._refreshAuditTrailGrid();
          } );

      // filters changed
      auditTrailGrid.on( 'changeFilters',
          function ()
          {
            me._refreshAuditTrailGrid();
          } );

      var auditTrailIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzAuditTrailIndexEnabled();

      auditTrailGrid.on( 'loadFailed',
          function ( operation )
          {
            auditTrailGrid.down( '#refreshButton' ).enable();
          } );

      auditTrailGrid.on( 'loadSuccess',
          function ( store, records )
          {
            auditTrailGrid.down( '#refreshButton' ).enable();
            auditTrailGrid.down( '#fetchPeriod' ).enable();
            auditTrailGrid.down( '#maxFetchSizeSelect' ).enable();
            // auditTrailGrid.down( '#fetchRelatedCheckbox' ).enable();

            // enable only if the index is enabled
            if ( auditTrailIndexEnabled )
            {
              auditTrailGrid.down( '#filterQueryText' ).enable();
            }

            // update empty text if there are no records
            var view = auditTrailGrid.getView();
            if ( Ext.isEmpty( records ) )
            {
              view.emptyText =
                  QDesk.Msg.get(
                      'QDesk.controller.command.scheduler.quartz.audit.InitializeSchedulerAuditTrailTab.emptyTextNoMatchingDataForScheduler' );

              view.refresh();
            }
          } );
    }
  },


  /**
   * Invoked to refresh the data in the scheduler audit trail grid.
   */
  _refreshAuditTrailGrid : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailGrid', {
      connectionCtx : me.connectionCtx
    } ).execute();
  }
} );
