/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Exports and saves the execution history.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.common.TabularDataExportFormat',

    'QDesk.util.EnumUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /*
   * Optional parameter with the JobDetail whose execution history should be exported.
   */
  jobDetail : undefined,

  /*
   * Optional parameter with the Trigger whose execution history should be exported.
   */
  triggerDetail : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var qdesk = {
      connectionCtx : me.connectionCtx
    };

    if ( me.jobDetail )
    {
      Ext.apply( qdesk, {
        jobDetail : me.jobDetail
      } );
    }

    if ( me.trigger )
    {
      Ext.apply( qdesk, {
        trigger : me.trigger
      } );
    }

    var window = Ext.widget( 'QuartzExportExecHistoryWindow', {
      qdesk : qdesk
    } );

    var formPanel = window.down( '#form' );

    // Export button
    formPanel.down( '#exportButton' ).on( 'click', function ()
    {
      me._exportData( window );
    } );

    window.show();
  },


  _exportData : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    // currently only CSV is supported
    var exportFormat = QDesk.model.common.TabularDataExportFormat.CSV;

    var selectedColumnsArray = formPanel.down( '#selectedColumns' ).getValue();
    var exportPeriod = formPanel.down( '#exportPeriod' ).getValue();
    var filterQueryText = formPanel.down( '#filterQueryText' ).getValue();

    var form = formPanel.getForm();

    var submitParams = {
      connectionId : me.connectionCtx.getConnectionId(),
      exportFormat : exportFormat,
      exportColumns : selectedColumnsArray.toString(),
      exportPeriod : Ext.encode( exportPeriod )
    };

    if ( !Ext.isEmpty( filterQueryText ) )
    {
      Ext.apply( submitParams, {
        filterQuery : filterQueryText
      } )
    }

    var actionUrl = QDesk.util.constant.ActionUrl.quartz.execHistoryActionExportForScheduler;

    if ( me.jobDetail )
    {
      actionUrl = QDesk.util.constant.ActionUrl.quartz.execHistoryActionExportForJob;

      Ext.apply( submitParams, {
        jobGroupName : me.jobDetail.get( 'group' ).name,
        jobName : me.jobDetail.get( 'name' )
      } );
    }

    if ( me.trigger )
    {
      actionUrl = QDesk.util.constant.ActionUrl.quartz.execHistoryActionExportForTrigger;

      Ext.apply( submitParams, {
        triggerGroupName : me.trigger.get( 'group' ).name,
        triggerName : me.trigger.get( 'name' )
      } );
    }

    form.doAction( 'ParamsSubmit', {
      url : actionUrl,
      params : submitParams,
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory.waitMsg',
          QDesk.util.EnumUtils.getTabularDataExportFormatName( exportFormat ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );

        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory.successMsg',
                  QDesk.util.EnumUtils.getTabularDataExportFormatName( exportFormat ) ),
              function ()
              {
                var exportFileHandle = json.data.handle;
                me._downloadExportData( exportFileHandle, exportFormat );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  _downloadExportData : function ( handle, exportFormat )
  {
    var downloadUrl = QDesk.util.constant.ActionUrl.quartz.execHistoryActionExportDownload +
        '&handle=' + handle +
        '&exportFormat=' + exportFormat;

    var iframe = Ext.getBody().createChild( {
      tag : 'iframe',
      width : 0,
      height : 0,
      frameborder : 0,
      src : downloadUrl,
      cls : Ext.baseCSSPrefix + 'hide-display'
    } );

    // give the backend 10 seconds to initiate the download and then silently drop the iframe
    Ext.defer( function ()
        {
          iframe.remove();
        },
        10000 );
  }
} );
