/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Execution History tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid',
    'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var execHistoryTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getExecHistoryTab(
        me.connectionCtx.getConnectionId() );

    me._bindEventHandlers( execHistoryTab );

    execHistoryTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Execution History tab to their respective handlers.
   */
  _bindEventHandlers : function ( execHistoryTab )
  {
    var me = this;
    me._bindEventHandlersExecHistoryGrid( execHistoryTab );
  },


  /**
   * Binds the jobs grid events to their handlers.
   */
  _bindEventHandlersExecHistoryGrid : function ( execHistoryTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    //var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    if ( licensedFeatures.ExecutionHistory && jvmAgentReady )
    {
      var execHistoryGrid = execHistoryTab.down( '#execHistoryGrid' );

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshSchedulerExecHistoryGrid();
          } );

      // export CSV button
      execHistoryGrid.down( '#exportCsvButton' ).on( 'click',
          function ()
          {
            me._exportExecHistory();
          } );

      // filters changed
      execHistoryGrid.on( 'changeFilters',
          function ()
          {
            me._refreshSchedulerExecHistoryGrid();
          } );

      // event fired by the actioncolumn in the JobExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( execHistoryId );
          } );

      var filterQueryText = execHistoryGrid.down( '#filterQueryText' );

      var execHistoryIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzExecHistoryIndexEnabled();

      if ( !execHistoryIndexEnabled )
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab.execHistoryIndexDisabledEmptyText' ) );
      }

      execHistoryGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            execHistoryGrid.down( '#refreshButton' ).disable();
            execHistoryGrid.down( '#fetchPeriod' ).disable();
            execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();
            filterQueryText.disable();
            execHistoryGrid.down( '#exportCsvButton' ).disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            execHistoryGrid.getSelectionModel().deselectAll();
          } );

      execHistoryGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            execHistoryGrid.down( '#refreshButton' ).enable();
            execHistoryGrid.down( '#fetchPeriod' ).enable();
            execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();

            // enable only if the exec history index is enabled in the agent
            if ( execHistoryIndexEnabled )
            {
              filterQueryText.enable();
            }

            if ( successful )
            {
              if ( jvmAgentReady )
              {
                execHistoryGrid.down( '#exportCsvButton' ).enable();
              }
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );
    }
  },


  /**
   * Invoked to refresh the data in the scheduler execution history grid.
   */
  _refreshSchedulerExecHistoryGrid : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid', {
      connectionCtx : me.connectionCtx
    } ).execute();
  },


  /**
   * Opens a window for exporting scheduler execution history data to a CSV file.
   * @private
   */
  _exportExecHistory : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory', {
      connectionCtx : me.connectionCtx
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   */
  _viewJobDataMap : function ( jobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      // exec history record can have null job data map !!!
      jobDataMap : jobDataMap == null ? [] : jobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param exception the exception data.
   */
  _viewException : function ( exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog', {
      connectionCtx : me.connectionCtx,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult', {
      connectionCtx : me.connectionCtx,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryUserData', {
      connectionCtx : me.connectionCtx,
      execHistoryId : execHistoryId
    } ).execute();
  }
} );
