/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured execution history log data.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.history.SaveExecHistoryLog',

    'QDesk.model.log.LoggingEventPriority',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.ViewLogWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the execution history record ID.
   */
  execHistoryId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.execHistoryId )
      Ext.Error.raise( 'Required config parameter missing: execHistoryId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionLog,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        execHistoryId : me.execHistoryId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._viewLogData( json );

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  _viewLogData : function ( json )
  {
    var me = this;

    var formattedLog = me._formatLogData( json.data );

    var window = Ext.widget( 'ViewLogWindow' );
    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );
    } );

    var saveButton = window.down( 'toolbar #saveButton' );
    saveButton.setDisabled( false );
    saveButton.on( 'click', function ()
    {
      me._saveLogData( window );
    } );

    var viewPanel = window.down( '#viewPanel' );
    viewPanel.setHtml( formattedLog );

    window.on( 'show', function ()
    {
      /*
       * Set focus on the editor so that the user can immediately use CTRL+A to
       * select all text in the editor. The delay is used to defer setting of the
       * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
       */
      QDesk.util.UxUtils.focus( viewPanel, false );

      if ( json.complete )
      {
        // all logging events loaded
      }
      else
      {
        // only the last N logging events loaded
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog.warnPartialDataShownMsg',
                json.returnedCount, json.totalCount ) );
      }
    } );

    window.show();
  },


  /**
   * Saves the job execution log data.
   * @private
   */
  _saveLogData : function ( window )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.SaveExecHistoryLog', {
      connectionCtx : me.connectionCtx,
      execHistoryId : me.execHistoryId,
      maskEl : window.getEl()
    } ).execute();
  },


  /**
   * Formats the specified list of logging events.
   * @param loggingEvents a list of logging event data.
   * @return the formatted logging events as an HTML string.
   * @private
   */
  _formatLogData : function ( loggingEvents )
  {
    var result = '<pre style="margin-bottom: 0px; margin-top: 0px;">';
    Ext.each( loggingEvents, function ( loggingEvent )
    {
      var priority = loggingEvent.priority;

      var priorityColor;
      if ( priority == QDesk.model.log.LoggingEventPriority.TRACE )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.trace' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.DEBUG )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.debug' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.INFO )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.info' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.WARN )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.warn' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.ERROR )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.error' )
      }

      var msg = '<span style="color: ' + priorityColor + '">';
      msg += Ext.String.htmlEncode( loggingEvent.message );
      msg += '</span><br/>';

      result += msg;
    } );
    result += '</pre>';

    return result;
  }
} );
