/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Info form in the Info tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.model.scheduler.SchedulerStatus',

    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProduct'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var infoTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getInfoTab(
        me.connectionCtx.getConnectionId() );

    QDesk.util.UxUtils.maskEl( infoTab.getEl(),
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.infoActionInfo,
      params : {
        connectionId : me.connectionCtx.getConnectionId()
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskEl( infoTab.getEl() );

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._loadConnectionInfoView( infoTab, json.data.scheduler );
            me._loadSchedulerInfoView( infoTab, json.data.scheduler, json.data.jvmAgentInfo );
            me._loadJvmAgentInfoTab( infoTab, json.data.jvmAgentInfo );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }
      }
    } );
  },


  _loadConnectionInfoView : function ( infoTab, schedulerData )
  {
    var connectionInfoForm = infoTab.down( '#connectionInfoForm' );

    connectionInfoForm.down( '#jmxProtocol' ).setValue( schedulerData.jmx.protocol );
    connectionInfoForm.down( '#jmxHost' ).setValue( schedulerData.jmx.host );
    connectionInfoForm.down( '#jmxPort' ).setValue( schedulerData.jmx.port );
    connectionInfoForm.down( '#jmxObjectName' ).setValue( schedulerData.jmx.objectName );
    connectionInfoForm.down( '#jmxServiceUrl' ).setValue( schedulerData.jmx.serviceUrl );
  },


  _loadSchedulerInfoView : function ( infoTab, schedulerData, jvmAgentInfo )
  {
    var schedulerInfoForm = infoTab.down( '#schedulerInfoForm' );

    schedulerInfoForm.down( '#status' ).setValue(
        '<span class="status' + schedulerData.status + '" data-qtip="' +
        Ext.String.htmlEncode( QDesk.Msg.get(
            'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipStatus' + schedulerData.status ) ) +
        '">' + schedulerData.displayStatus.toUpperCase() + '</span>' );

    schedulerInfoForm.down( '#name' ).setValue( schedulerData.name );
    schedulerInfoForm.down( '#instanceId' ).setValue( schedulerData.instanceId );
    schedulerInfoForm.down( '#version' ).setValue( schedulerData.displayVersion );
    schedulerInfoForm.down( '#threadPoolClassName' ).setValue( schedulerData.threadPoolClassName );
    schedulerInfoForm.down( '#threadPoolSize' ).setValue( schedulerData.threadPoolSize );
    schedulerInfoForm.down( '#jobStoreClassName' ).setValue( schedulerData.jobStoreClassName );
    schedulerInfoForm.down( '#monitoringUrl' ).setValue( schedulerData.monitoringUrl );

    var dateInitializedField = schedulerInfoForm.down( '#dateInitialized' );
    var dateStartedField = schedulerInfoForm.down( '#dateStarted' );
    var datePausedField = schedulerInfoForm.down( '#datePaused' );
    var dateStoppedField = schedulerInfoForm.down( '#dateStopped' );

    var jobCountField = schedulerInfoForm.down( '#jobCount' );
    var triggerCountField = schedulerInfoForm.down( '#triggerCount' );

    var classLoaderInfoField = schedulerInfoForm.down( '#classLoaderInfo' );

    var publicApiInfoFieldSet = schedulerInfoForm.down( '#publicApiInfo' );
    var publicApiDetectedField = publicApiInfoFieldSet.down( '#publicApiDetected' );

    var publicApiVersionField = publicApiInfoFieldSet.down( '#publicApiVersion' );
    var publicApiCompatibleVersionField = publicApiInfoFieldSet.down( '#publicApiCompatibleVersion' );

    var publicApiClassLoaderInfoField = publicApiInfoFieldSet.down( '#publicApiClassLoaderInfo' );
    var publicApiLoadedCorrectlyField = publicApiInfoFieldSet.down( '#publicApiLoadedCorrectly' );

    var availableJobClassesGrid = schedulerInfoForm.down( '#availableJobClasses' );

    var jobListenersGrid = schedulerInfoForm.down( '#jobListeners' );
    var triggerListenersGrid = schedulerInfoForm.down( '#triggerListeners' );
    var schedulerListenersGrid = schedulerInfoForm.down( '#schedulerListeners' );

    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;
    if ( licensedProductEdition === QDesk.model.license.ProductEdition.LITE )
    {
      // hide all of the extra fields
      dateInitializedField.setVisible( false );
      dateStartedField.setVisible( false );
      datePausedField.setVisible( false );
      dateStoppedField.setVisible( false );

      classLoaderInfoField.setVisible( false );
      publicApiInfoFieldSet.setVisible( false );
      availableJobClassesGrid.setVisible( false );
      jobListenersGrid.setVisible( false );
      triggerListenersGrid.setVisible( false );
      schedulerListenersGrid.setVisible( false );
    }
    else
    {
      var jvmAgentDetected = jvmAgentInfo.detected;
      var jvmAgentReady = jvmAgentInfo.ready;
      var publicApiReady = schedulerData.publicApiInfo.ready;

      if ( jvmAgentDetected )
      {
        classLoaderInfoField.setVisible( true );
        classLoaderInfoField.setValue( schedulerData.classLoaderInfo );
      }

      if ( jvmAgentReady && publicApiReady )
      {
        dateInitializedField.setValue( schedulerData.dateInitialized );
        dateInitializedField.setVisible( true );

        if ( schedulerData.status === QDesk.model.scheduler.SchedulerStatus.STARTED )
        {
          dateStartedField.setValue( schedulerData.dateStarted );
          dateStartedField.setVisible( true );
        }
        else
        {
          dateStartedField.setVisible( false );
        }

        if ( schedulerData.status === QDesk.model.scheduler.SchedulerStatus.PAUSED )
        {
          datePausedField.setValue( schedulerData.datePaused );
          datePausedField.setVisible( true );
        }
        else
        {
          datePausedField.setVisible( false );
        }


        if ( schedulerData.status === QDesk.model.scheduler.SchedulerStatus.STOPPED )
        {
          dateStoppedField.setValue( schedulerData.dateStopped );
          dateStoppedField.setVisible( true );
        }
        else
        {
          dateStoppedField.setVisible( false );
        }

        jobCountField.setValue( schedulerData.jobCount );
        jobCountField.setVisible( true );

        triggerCountField.setValue( schedulerData.triggerCount );
        triggerCountField.setVisible( true );

        availableJobClassesGrid.setVisible( true );
        availableJobClassesGrid.getStore().loadData( schedulerData.availableJobClasses );

        jobListenersGrid.setVisible( true );
        jobListenersGrid.getStore().loadData( schedulerData.jobListeners );

        triggerListenersGrid.setVisible( true );
        triggerListenersGrid.getStore().loadData( schedulerData.triggerListeners );

        schedulerListenersGrid.setVisible( true );
        schedulerListenersGrid.getStore().loadData( schedulerData.schedulerListeners );
      }
      else
      {
        // public API is not ready
        dateInitializedField.setVisible( false );
        dateStartedField.setVisible( false );
        datePausedField.setVisible( false );
        dateStoppedField.setVisible( false );

        jobCountField.setVisible( false );
        triggerCountField.setVisible( false );

        availableJobClassesGrid.setVisible( false );

        jobListenersGrid.setVisible( false );
        triggerListenersGrid.setVisible( false );
        schedulerListenersGrid.setVisible( false );
      }

      /*
       * QuartzDesk Public Info fields - shown if the agent is detected.
       */
      publicApiInfoFieldSet.setVisible( true );
      if ( jvmAgentDetected )
      {
        //
        // QuartzDesk Public API detected flag
        //
        if ( schedulerData.publicApiInfo.detected )
        {
          publicApiDetectedField.setValue(
              '<span class="yes" data-qtip="' +
              Ext.String.htmlEncode( QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiDetected' ) ) +
              '">' + QDesk.Msg.get( 'QDesk.yes' ).toUpperCase() + '</span>' );

          publicApiDetectedField.setVisible( true );

          //
          // QuartzDesk Public API version
          //
          publicApiVersionField.setValue( schedulerData.publicApiInfo.version );
          publicApiVersionField.setVisible( true );

          //
          // QuartzDesk Public API compatible version flag
          //
          if ( schedulerData.publicApiInfo.compatibleVersion )
          {
            publicApiCompatibleVersionField.setValue(
                '<span class="yes" data-qtip="' +
                Ext.String.htmlEncode( QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiCompatibleVersion',
                    QDesk.util.FormatUtils.formatVersion( schedulerData.publicApiInfo.version ),
                    QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.version ) ) ) +
                '">' + QDesk.Msg.get( 'QDesk.yes' ).toUpperCase() + '</span>' );
          }
          else
          {
            publicApiCompatibleVersionField.setValue(
                '<span class="no" data-qtip="' +
                Ext.String.htmlEncode( QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiIncompatibleVersion',
                    QDesk.util.FormatUtils.formatVersion( schedulerData.publicApiInfo.version ),
                    QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.version ) ) ) +
                '">' + QDesk.Msg.get( 'QDesk.no' ).toUpperCase() + '</span>' );
          }
          publicApiCompatibleVersionField.setVisible( true );

          //
          // QuartzDesk Public API loaded correctly flag
          //
          if ( schedulerData.publicApiInfo.loadedCorrectly )
          {
            publicApiLoadedCorrectlyField.setValue(
                '<span class="yes" data-qtip="' +
                Ext.String.htmlEncode( QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiLoadedCorrectly' ) ) +
                '">' + QDesk.Msg.get( 'QDesk.yes' ).toUpperCase() + '</span>' );
          }
          else
          {
            publicApiLoadedCorrectlyField.setValue(
                '<span class="no" data-qtip="' +
                Ext.String.htmlEncode( QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiNotLoadedCorrectly' ) ) +
                '">' + QDesk.Msg.get( 'QDesk.no' ).toUpperCase() + '</span>' );
          }
          publicApiLoadedCorrectlyField.setVisible( true );

          //
          // QuartzDesk Public API class loader info
          //
          publicApiClassLoaderInfoField.setValue( schedulerData.publicApiInfo.classLoaderInfo );
          publicApiClassLoaderInfoField.setVisible( true );
        }
        else
        {
          publicApiDetectedField.setValue(
              '<span class="no" data-qtip="' +
              Ext.String.htmlEncode( QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiNotDetected' ) ) +
              '">' + QDesk.Msg.get( 'QDesk.no' ).toUpperCase() + '</span>' );

          publicApiDetectedField.setVisible( true );
          publicApiVersionField.setVisible( false );
          publicApiCompatibleVersionField.setVisible( false );
          publicApiLoadedCorrectlyField.setVisible( false );
          publicApiClassLoaderInfoField.setVisible( false );
        }
      }
      else
      {
        // agent not detected - we cannot obtain QuartzDesk Public API details
        publicApiDetectedField.setValue(
            '<span class="no" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipPublicApiNotDetectedAgentNotDetected' ) ) +
            '">' + QDesk.Msg.get( 'QDesk.no' ).toUpperCase() + '</span>' );

        publicApiDetectedField.setVisible( true );
      }
    }
  },


  _loadJvmAgentInfoTab : function ( infoTab, jvmAgentInfo )
  {
    var jvmAgentInfoForm = infoTab.down( '#jvmAgentInfoForm' );

    var detectedField = jvmAgentInfoForm.down( '#detected' );
    var nameAndVersionField = jvmAgentInfoForm.down( '#nameAndVersion' );
    var versionField = jvmAgentInfoForm.down( '#version' );
    var editionField = jvmAgentInfoForm.down( '#edition' );
    var localeField = jvmAgentInfoForm.down( '#locale' );
    var timeZoneField = jvmAgentInfoForm.down( '#timeZone' );

    var classLoaderInfoField = jvmAgentInfoForm.down( '#classLoaderInfo' );
    var configurationGrid = jvmAgentInfoForm.down( '#configuration' );

    if ( jvmAgentInfo.detected )
    {
      detectedField.setValue(
          '<span class="yes" data-qtip="' +
          Ext.String.htmlEncode( QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipJvmAgentDetected' ) ) +
          '">' + QDesk.Msg.get( 'QDesk.yes' ).toUpperCase() + '</span>' );

      detectedField.setVisible( true );

      if ( jvmAgentInfo.nameAndVersion )
      {
        nameAndVersionField.setVisible( true );
        nameAndVersionField.setValue( jvmAgentInfo.nameAndVersion );
      }

      /*
       * Agent version.
       */
      if ( jvmAgentInfo.displayVersion )
      {
        versionField.setVisible( true );

        var versionCompatibilityTooltip;
        if ( jvmAgentInfo.compatibleVersion )
        {
          versionCompatibilityTooltip = QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipAgentCompatibleVersion',
              QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.version ),
              QDesk.util.FormatUtils.formatVersion(
                  QDesk.util.license.LicensedProduct.productVersion ) );

          versionField.setValue(
              jvmAgentInfo.displayVersion + '<span style="margin-left: 15px" class="compatible" data-qtip="' +
              Ext.String.htmlEncode( versionCompatibilityTooltip ) +
              '">' + QDesk.Msg.get( 'QDesk.compatible' ).toUpperCase() + '</span>' );
        }
        else
        {
          versionCompatibilityTooltip = QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipAgentIncompatibleVersion',
              QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.version ),
              QDesk.util.FormatUtils.formatVersion(
                  QDesk.util.license.LicensedProduct.productVersion ) );

          versionField.setValue(
              jvmAgentInfo.displayVersion + '<span style="margin-left: 15px;" class="incompatible" data-qtip="' +
              Ext.String.htmlEncode( versionCompatibilityTooltip ) +
              '">' + QDesk.Msg.get( 'QDesk.incompatible' ).toUpperCase() + '</span>' );
        }
      }

      /*
       * Agent edition.
       */
      if ( jvmAgentInfo.edition )
      {
        editionField.setVisible( true );

        var editionCompatibilityTooltip;
        if ( jvmAgentInfo.compatibleEdition )
        {
          editionCompatibilityTooltip = QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipAgentCompatibleEdition',
              jvmAgentInfo.edition, QDesk.util.license.LicensedProduct.productEdition );

          editionField.setValue(
              jvmAgentInfo.edition + '<span style="margin-left: 15px" class="compatible" data-qtip="' +
              Ext.String.htmlEncode( editionCompatibilityTooltip ) +
              '">' + QDesk.Msg.get( 'QDesk.compatible' ).toUpperCase() + '</span>' );
        }
        else
        {
          editionCompatibilityTooltip = QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipAgentIncompatibleEdition',
              jvmAgentInfo.edition, QDesk.util.license.LicensedProduct.productEdition );

          editionField.setValue(
              jvmAgentInfo.edition + '<span style="margin-left: 15px" class="incompatible" data-qtip="' +
              Ext.String.htmlEncode( editionCompatibilityTooltip ) +
              '">' + QDesk.Msg.get( 'QDesk.incompatible' ).toUpperCase() + '</span>' );
        }
      }

      if ( jvmAgentInfo.locale )
      {
        localeField.setVisible( true );
        localeField.setValue( jvmAgentInfo.locale );
      }

      if ( jvmAgentInfo.timeZone )
      {
        timeZoneField.setVisible( true );
        timeZoneField.setValue( jvmAgentInfo.timeZone );
      }

      if ( jvmAgentInfo.classLoaderInfo )
      {
        classLoaderInfoField.setVisible( true );
        classLoaderInfoField.setValue( jvmAgentInfo.classLoaderInfo );
      }

      if ( jvmAgentInfo.configuration )
      {
        configurationGrid.setVisible( true );
        configurationGrid.getStore().loadData( jvmAgentInfo.configuration );
      }
    }
    else
    {
      detectedField.setValue(
          '<span class="no" data-qtip="' +
          Ext.String.htmlEncode( QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab.tooltipAgentNotDetected' ) ) +
          '">' + QDesk.Msg.get( 'QDesk.no' ).toUpperCase() + '</span>' );

      detectedField.setVisible( true );

      nameAndVersionField.setVisible( false );
      versionField.setVisible( false );
      editionField.setVisible( false );
      localeField.setVisible( false );
      timeZoneField.setVisible( false );

      classLoaderInfoField.setVisible( false );
      configurationGrid.setVisible( false );
    }
  }
} );
