/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Planned Job Executions tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.ViewJobDataMap',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsGrid',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var plannedJobExecutionsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getPlannedJobExecutionsTab(
        me.connectionCtx.getConnectionId() );

    me._bindEventHandlers( plannedJobExecutionsTab );

    plannedJobExecutionsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified Planned Job Executions tab to their respective handlers.
   */
  _bindEventHandlers : function ( plannedJobExecutionsTab )
  {
    var me = this;
    me._bindEventHandlersPlannedJobExecutionsGrid( plannedJobExecutionsTab );
  },


  /**
   * Binds the Planned Job Executions grid events to their handlers.
   */
  _bindEventHandlersPlannedJobExecutionsGrid : function ( plannedJobExecutionsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    if ( licensedFeatures.PlannedJobExecutions && jvmAgentReady && publicApiReady )
    {
      var jobsIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzJobsIndexEnabled();

      var plannedJobExecutionsGrid = plannedJobExecutionsTab.down( '#plannedJobExecutionsGrid' );

      var timePeriod = plannedJobExecutionsGrid.down( '#timePeriod' );
      var filterQueryText = plannedJobExecutionsGrid.down( '#filterQueryText' );
      var submitButton = plannedJobExecutionsGrid.down( '#submitButton' );

      // filters changed (may be invalid)
      plannedJobExecutionsGrid.on( 'changeFilters',
          function ()
          {
            if ( plannedJobExecutionsGrid.filtersValid() )
            {
              if ( jobsIndexEnabled )
              {
                filterQueryText.enable();
              }
              submitButton.enable();

              plannedJobExecutionsGrid.empty( QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextInitial' ) );
            }
            else
            {
              // filters are not valid
              filterQueryText.disable();
              submitButton.disable();

              plannedJobExecutionsGrid.empty( QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextInvalidFilters' ) );
            }
          } );

      plannedJobExecutionsGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      plannedJobExecutionsGrid.down( '#triggerJobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      plannedJobExecutionsGrid.down( '#mergedJobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      plannedJobExecutionsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onPlannedJobExecutionUnselected( plannedJobExecutionsTab );
        }
        else
        {
          me._onPlannedJobExecutionSelected( selectedRecords[ 0 ], plannedJobExecutionsTab );
        }
      } );

      submitButton.on( 'click',
          function ()
          {
            me._refreshPlannedJobExecutionsGrid();
          } );

      if ( !jobsIndexEnabled )
      {
        filterQueryText.disable();
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab.jobsIndexDisabledEmptyText' ) );
      }

      plannedJobExecutionsGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        store.removeAll();

        timePeriod.disable();
        filterQueryText.disable();
        submitButton.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        plannedJobExecutionsGrid.getSelectionModel().deselectAll();
      } );

      plannedJobExecutionsGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            timePeriod.enable();

            // enable only if the jobs index is enabled in the agent
            if ( jobsIndexEnabled )
            {
              filterQueryText.enable();
            }

            submitButton.enable();

            if ( successful )
            {
              var response = operation.getResponse();

              // if the retrieved planned job executions data is not complete, show a warning
              if ( response.responseJson && !response.responseJson.complete )
              {
                QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab.warningLoadedDataNotComplete',
                    1000 ) );
              }
            }
            else
            {
              //store.removeAll();  // moved to the beforeload handler

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );

      /* PlannedJobChainsGrid context menu actions */
      //plannedJobExecutionsGrid.on( 'cloneJobChain', me._cloneJobChain, me );
    }
  },


  /**
   * Invoked when a planned job execution record in the PlannedJobExecutionsGrid in the PlannedJobExecutionsTab
   * is unselected.
   * @param jobChain a QDesk.model.scheduler.quartz.QuartzPlannedJobExecution instance representing the selected
   * planned job execution.
   * @param plannedJobExecutionsTab the Planned Job Executions tab.
   */
  _onPlannedJobExecutionSelected : function ( jobChain, plannedJobExecutionsTab )
  {
    var me = this;

    //var plannedJobExecutionsGrid = plannedJobExecutionsTab.down( '#plannedJobExecutionsGrid' );
  },


  /**
   * Invoked when a planned job execution record in the PlannedJobExecutionsGrid in the PlannedJobExecutionsTab
   * is unselected.
   * @param plannedJobExecutionsTab the Planned Job Executions tab.
   */
  _onPlannedJobExecutionUnselected : function ( plannedJobExecutionsTab )
  {
    var me = this;

    //var plannedJobExecutionsGrid = plannedJobExecutionsTab.down( '#plannedJobExecutionsGrid' );
  },


  /**
   * Invoked to refresh the data in the planned job executions grid.
   */
  _refreshPlannedJobExecutionsGrid : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsGrid', {
      connectionCtx : me.connectionCtx
    } ).execute();
  },

  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   */
  _viewJobDataMap : function ( jobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      // exec history record can have null job data map !!!
      jobDataMap : jobDataMap == null ? [] : jobDataMap
    } ).execute();
  }
} );
