/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Misfired Triggers tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.misfire.InitializeSchedulerMisfiredTriggersTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersGrid',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var misfiredTriggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getMisfiredTriggersTab(
        me.connectionCtx.getConnectionId() );

    me._bindEventHandlers( misfiredTriggersTab );

    misfiredTriggersTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Misfired Triggers tab to their respective handlers.
   */
  _bindEventHandlers : function ( misfiredTriggersTab )
  {
    var me = this;
    me._bindEventHandlersMisfiredTriggersGrid( misfiredTriggersTab );
  },


  /**
   * Binds the Misfired Triggers grid events to their handlers.
   */
  _bindEventHandlersMisfiredTriggersGrid : function ( misfiredTriggersTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    //var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    if ( licensedFeatures.MisfiredTriggers && jvmAgentReady )
    {
      var misfiredTriggersGrid = misfiredTriggersTab.down( '#misfiredTriggersGrid' );

      // refresh button
      misfiredTriggersGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshMisfiredTriggersGrid();
          } );

      // filters changed
      misfiredTriggersGrid.on( 'changeFilters',
          function ()
          {
            me._refreshMisfiredTriggersGrid();
          } );

      var filterQueryText = misfiredTriggersGrid.down( '#filterQueryText' );

      var misfiredTriggersIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzMisfiredTriggersIndexEnabled();

      if ( !misfiredTriggersIndexEnabled )
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.misfire.InitializeSchedulerMisfiredTriggersTab.misfiredTriggersIndexDisabledEmptyText' ) );
      }

      misfiredTriggersGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            misfiredTriggersGrid.down( '#refreshButton' ).disable();
            misfiredTriggersGrid.down( '#fetchPeriod' ).disable();
            misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            misfiredTriggersGrid.getSelectionModel().deselectAll();
          } );

      misfiredTriggersGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            misfiredTriggersGrid.down( '#refreshButton' ).enable();
            misfiredTriggersGrid.down( '#fetchPeriod' ).enable();
            misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).enable();

            // enable only if the exec history index is enabled in the agent
            if ( misfiredTriggersIndexEnabled )
            {
              filterQueryText.enable();
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );
    }
  },


  /**
   * Invoked to refresh the data in the misfired triggers grid.
   */
  _refreshMisfiredTriggersGrid : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersGrid', {
      connectionCtx : me.connectionCtx
    } ).execute();
  }
} );
