/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Scheduler Statistics tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.work.scheduler.quartz.stats.ExecCountChart',
    'QDesk.view.work.scheduler.quartz.stats.ExecTimeChart',

    'QDesk.view.work.scheduler.quartz.stats.MostFrequentlyExecutedJobsChart',
    'QDesk.view.work.scheduler.quartz.stats.FastestExecutedJobsChart',
    'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedJobsChart',

    'QDesk.view.work.scheduler.quartz.stats.MostFrequentlyExecutedTriggersChart',
    'QDesk.view.work.scheduler.quartz.stats.FastestExecutedTriggersChart',
    'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedTriggersChart'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var statisticsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getStatisticsTab(
          me.connectionCtx.getConnectionId() );

      var statisticsPanel = statisticsTab.down( '#statisticsPanel' );

      var statisticsType = statisticsPanel.getStatisticsType();
      var periodLength = statisticsPanel.getPeriodLength();
      var periodStart = statisticsPanel.getPeriodStart();

      /*
       * Remove chart description text and remove the chart component.
       */
      var chartDescriptionPanel = statisticsPanel.down( '#chartDescription' );
      chartDescriptionPanel.setVisible( false );
      chartDescriptionPanel.setMessage( '' );

      statisticsPanel.down( '#chartContainer' ).removeAll();

      // disable all fields in the statistics' panel toolbar
      var toolbar = statisticsPanel.down( 'toolbar' );
      Ext.each( Ext.ComponentQuery.query( 'button, field', toolbar ),
          function ( item )
          {
            item.disable();
          }
      );

      if ( statisticsType && periodLength )
      {
        if ( periodStart == null )
        {
          me._periodsNotAvailable( statisticsPanel, statisticsType, periodLength );
        }
        else
        {
          me._renderChartAndDescription( statisticsPanel, statisticsType, periodLength, periodStart );
        }
      }
    }
  },


  _periodsNotAvailable : function ( statisticsPanel, statisticsType, periodLength )
  {
    var me = this;

    var toolbar = statisticsPanel.down( 'toolbar' );

    // enable the refresh button + all fields in the statistics' panel toolbar but the period selection controls
    Ext.each( Ext.ComponentQuery.query( '#refreshButton, field', toolbar ),
        function ( item )
        {
          if ( !Ext.String.endsWith( item.getItemId(), 'PeriodSelect' ) )
            item.enable();
        }
    );

    statisticsPanel._reloadAvailablePeriods();
  },


  _renderChartAndDescription : function ( statisticsPanel, statisticsType, periodLength, periodStart )
  {
    var me = this;

    var toolbar = statisticsPanel.down( 'toolbar' );

    // enable the refresh button + all fields in the statistics' panel toolbar
    Ext.each( Ext.ComponentQuery.query( '#refreshButton, field', toolbar ),
        function ( item )
        {
          item.enable();
        }
    );

    var chart;
    var chartDescription;

    switch ( statisticsType )
    {
      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.JOB_EXEC_COUNT:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.ExecCountChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.JOB_EXEC_TIME:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.ExecTimeChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.TOP_MOST_FREQ_EXEC_JOBS:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.MostFrequentlyExecutedJobsChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType,
                QDesk.ApplicationConfig.get( 'quartz.statistics.scheduler.topMostFrequentlyExecutedJobs.maxResults' ) );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.TOP_FASTEST_EXEC_JOBS:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.FastestExecutedJobsChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType,
                QDesk.ApplicationConfig.get( 'quartz.statistics.scheduler.topFastestExecutedJobs.maxResults' ) );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.TOP_SLOWEST_EXEC_JOBS:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedJobsChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType,
                QDesk.ApplicationConfig.get( 'quartz.statistics.scheduler.topSlowestExecutedJobs.maxResults' ) );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.TOP_MOST_FREQ_EXEC_TRIGGERS:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.MostFrequentlyExecutedTriggersChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType,
                QDesk.ApplicationConfig.get(
                    'quartz.statistics.scheduler.topMostFrequentlyExecutedTriggers.maxResults' ) );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.TOP_FASTEST_EXEC_TRIGGERS:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.FastestExecutedTriggersChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType,
                QDesk.ApplicationConfig.get( 'quartz.statistics.scheduler.topFastestExecutedTriggers.maxResults' ) );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.TOP_SLOWEST_EXEC_TRIGGERS:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedTriggersChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );

        chartDescription =
            QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType.description.' +
                statisticsType,
                QDesk.ApplicationConfig.get( 'quartz.statistics.scheduler.topSlowestExecutedTriggers.maxResults' ) );
        break;

      default:
        Ext.Error.raise( 'Unexpected scheduler statistics type: ' + statisticsType );
    }

    var store = chart.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.schedulerStatisticsActionChartData;

    store.getProxy().extraParams = {
      connectionId : me.connectionCtx.getConnectionId(),
      statisticsType : statisticsType,
      periodLength : periodLength,
      periodStart : periodStart
    };

    me.mask(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab.maskLoadMsg' ) );

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          var chartDescriptionPanel = statisticsPanel.down( '#chartDescription' );
          chartDescriptionPanel.setMessage( chartDescription );
          chartDescriptionPanel.setVisible( true );

          statisticsPanel.down( '#chartContainer' ).add( chart );

          // enable remaining buttons (eg. save chart) that have been disabled
          Ext.each( Ext.ComponentQuery.query( 'button', toolbar ),
              function ( item )
              {
                item.enable();
              }
          );

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
