/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the specified ServiceEndpointsTab panel.
 */
Ext.define( 'QDesk.controller.command.system.ws.InitializeServiceEndpointsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.ViewError',
    'QDesk.controller.command.system.ws.RefreshServiceEndpointsGrid',
    'QDesk.controller.command.system.ws.StartServiceEndpoint',
    'QDesk.controller.command.system.ws.StopServiceEndpoint',
    'QDesk.controller.command.system.ws.ResetServiceEndpointStatistics'
  ],


  /**
   * Required config parameter with the ServiceEndpointsTab panel.
   */
  panel : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.panel )
      Ext.Error.raise( 'Required config parameter missing: panel' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me._bindEventHandlers( me.panel );

    me.panel.setInitialized( true );
  },


  /**
   * Binds the service endpoints tab events to their handlers.
   */
  _bindEventHandlers : function ( serviceEndpointsTab )
  {
    var me = this;
    me._bindEventHandlersGrid( serviceEndpointsTab );
  },


  /**
   * Binds the service endpoints grid events to their handlers.
   */
  _bindEventHandlersGrid : function ( serviceEndpointsTab )
  {
    var me = this;
    var serviceEndpointsGrid = serviceEndpointsTab.down( 'ServiceEndpointsGrid' );

    serviceEndpointsGrid.on( 'refresh',
        function ()
        {
          me._refreshServiceEndpointsGrid( serviceEndpointsTab );
        } );

    serviceEndpointsGrid.on( 'startEndpoint',
        function ( serviceEndpoint )
        {
          me._startServiceEndpoint( serviceEndpointsTab, serviceEndpoint );
        } );

    serviceEndpointsGrid.on( 'stopEndpoint',
        function ( serviceEndpoint )
        {
          me._stopServiceEndpoint( serviceEndpointsTab, serviceEndpoint );
        } );

    serviceEndpointsGrid.on( 'resetEndpointStatistics',
        function ( serviceEndpoint )
        {
          me._resetServiceEndpointStatistics( serviceEndpointsTab, serviceEndpoint );
        } );

    serviceEndpointsGrid.getView().on( 'expandbody',
        function ( rowDom, record )
        {
          var rowEl = Ext.get( rowDom );
          var imageEls = rowEl.query( 'img[class=clickable][data-operation-id]', false );
          Ext.each( imageEls,
              function ( imageEl )
              {
                if ( imageEl.hasListeners.click )
                {
                  // image already has a click listener, do not add a new one
                }
                else
                {
                  var operationId = parseInt( imageEl.getAttribute( 'data-operation-id' ) );

                  // find the error text in the ServiceEndpoint record
                  var errorText = null;
                  var operations = record.get( 'operation' );
                  Ext.each( operations,
                      function ( operation )
                      {
                        if ( operation.id == operationId )
                          errorText = operation.lastErrorDesc;
                      } );

                  imageEl.on( 'click',
                      function ()
                      {
                        if ( !Ext.isEmpty( errorText ) )
                        {
                          me._viewServiceEndpointOperationError( operationId, errorText );
                        }
                      } );
                }
              } );
        }
    );
  },


  _refreshServiceEndpointsGrid : function ( serviceEndpointsTab )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.system.ws.RefreshServiceEndpointsGrid', {
      panel : serviceEndpointsTab
    } ).execute();
  },


  _startServiceEndpoint : function ( serviceEndpointsTab, serviceEndpoint )
  {
    var me = this;

    var service = serviceEndpointsTab.getService();

    Ext.create( 'QDesk.controller.command.system.ws.StartServiceEndpoint', {
      service : service,
      serviceEndpoint : serviceEndpoint,
      /*
       * Upon success refresh the service endpoints tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.system.ws.RefreshServiceEndpointsGrid', {
          panel : serviceEndpointsTab
        } ).execute();
      }
    } ).execute();
  },


  _stopServiceEndpoint : function ( serviceEndpointsTab, serviceEndpoint )
  {
    var me = this;

    var service = serviceEndpointsTab.getService();

    Ext.create( 'QDesk.controller.command.system.ws.StopServiceEndpoint', {
      service : service,
      serviceEndpoint : serviceEndpoint,
      /*
       * Upon success refresh the service endpoints tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.system.ws.RefreshServiceEndpointsGrid', {
          panel : serviceEndpointsTab
        } ).execute();
      }
    } ).execute();
  },


  _resetServiceEndpointStatistics : function ( serviceEndpointsTab, serviceEndpoint )
  {
    var me = this;

    var service = serviceEndpointsTab.getService();

    Ext.create( 'QDesk.controller.command.system.ws.ResetServiceEndpointStatistics', {
      service : service,
      serviceEndpoint : serviceEndpoint,
      /*
       * Upon success refresh the service endpoints tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.system.ws.RefreshServiceEndpointsGrid', {
          panel : serviceEndpointsTab
        } ).execute();
      }
    } ).execute();
  },


  _viewServiceEndpointOperationError : function ( operationId, errorText )
  {
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : errorText
    } ).execute();
  }
} );
