/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.util.UxUtils', {
  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.model.gui.ServerErrorType',

    'QDesk.util.FormatUtils',
    'QDesk.util.constant.Const',

    'QDesk.view.work.ExceptionErrorWindow'
  ],

  singleton : true,

  /**
   * Handler function that can be used to handle all errors
   * generated by the ExtJS framework and the application with
   * the Ext.Error.raise function like so:
   * <pre>
   *    Ext.Error.raise('JSON object not found')
   * </pre>
   * @param err an error object that is guaranteed to contain the
   * 'msg' property with the error message.
   * @param callback an optional call-back function that will be invoked when
   * the user closes the error dialog.
   */
  handleError : function ( err, callback )
  {
    Ext.log( {
      msg : err.msg,
      level : 'error',
      dump : err,
      stack : true
    } );

    Ext.MessageBox.show( {
      title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleInternalError' ),
      msg : err.msg,
      buttons : Ext.MessageBox.OK,
      icon : Ext.MessageBox.ERROR,
      fn : callback
    } );

    // ExtJS.raise() will not rethrow the error.
    return Ext.Error.ignore;
  },


  showExtJsAjaxError : function ( response, callback )
  {
    var me = this;

    if ( response && response.responseJson )
    {
      // application errors
      me.showJsonResponseErrors( response.responseJson, callback );
    }
    else
    {
      /*
       * Failed ExtJS ajax response contains the error details as opposed to the failed ExtJS failed operation
       * where the response (operation.getResponse()) is null.
       */
      var status = response.status;
      var statusText = response.statusText;

      QDesk.util.UxUtils._showResponseError( status, statusText, callback );
    }
  },


  showExtJsOperationError : function ( operation, callback )
  {
    var me = this;

    var response = operation.getResponse();
    if ( response && response.responseJson )
    {
      // application errors
      me.showJsonResponseErrors( response.responseJson, callback );
    }
    else
    {
      /*
       * Failed ExtJS operations have response == null and the error details must be extracted from the
       * error object has the following structure:
       *
       * {
       *   status : 0,
       *   statusText : 'communication failure'
       *   response : { ...}
       * }
       *
       */
      var error = operation.getError();
      QDesk.util.UxUtils._showResponseError( error.status, error.statusText, callback );
    }
  },


  showExtJsActionError : function ( action, callback )
  {
    var me = this;

    /*
     * Failure type returned when server side processing fails and the result's success property is set to false.
     */
    if ( action.failureType == Ext.form.action.Action.SERVER_INVALID )
    {
      me.showJsonResponseErrors( action.result, callback );
    }

    /*
     * Failure type returned when the response's success property is set to false, or no field values are returned
     * in the response's data property.
     */
    else if ( action.failureType == Ext.form.action.Action.LOAD_FAILURE )
    {
      me.showJsonResponseErrors( action.result, callback );
    }

    /*
     * Failure type returned when a communication error happens when attempting to send a request to the remote server.
     * The response may be examined to provide further information.
     */
    else if ( action.failureType == Ext.form.action.Action.CONNECT_FAILURE )
    {
      me.showErrorDialog( QDesk.Msg.get( 'QDesk.util.UxUtils.actionErrorConnectFailureMsg' ), callback );
    }

    /*
     * Failure type returned when client side validation of the Form fails thus aborting a submit action. Client side
     * validation is performed unless Ext.form.action.Submit#clientValidation is explicitly set to false.
     */
    else if ( action.failureType == Ext.form.action.Action.CLIENT_INVALID )
    {
      me.showErrorDialog( QDesk.Msg.get( 'QDesk.util.UxUtils.actionErrorClientInvalidMsg' ), callback );
    }

    /*
     * All other types of errors. This section should never be reached.
     */
    else
    {
      me.showErrorDialog( QDesk.Msg.get( 'QDesk.util.UxUtils.actionErrorUnexpectedMsg' ), callback )
    }
  },


  /**
   * Shows an error dialog displaying the errors from the specified application
   * Ajax response. For the structure of the errorResp parameter, refer to the
   * AjaxActionBase class.
   * @param responseJson an Ajax error response.
   * @param callback an optional call-back function that will be invoked when
   * the user closes the error dialog.
   */
  showJsonResponseErrors : function ( responseJson, callback )
  {
    var errorType = responseJson.errorType;
    var errorTitle;

    var errorMessages = '';

    switch ( errorType )
    {
      case QDesk.model.gui.ServerErrorType.APPLICATION:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleApplicationError' );
        break;

      case QDesk.model.gui.ServerErrorType.EXCEPTION:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleExceptionError' );
        break;

      case QDesk.model.gui.ServerErrorType.VALIDATION:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleValidationError' );
        break;

      case QDesk.model.gui.ServerErrorType.UNRECOGNIZED_EVENT:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleUnrecognizedEventError' );
        break;

      case QDesk.model.gui.ServerErrorType.AUTHENTICATION:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleAuthenticationError' );
        break;

      case QDesk.model.gui.ServerErrorType.AUTHORIZATION:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleAuthorizationError' );
        break;

      default:
        errorTitle = QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleError' );
        break;
    }

    /**
     * Server-side errors with exceptions are shown in a window that shows the exception stack trace.
     */
    if ( errorType === QDesk.model.gui.ServerErrorType.EXCEPTION )
    {
      /*
       * result.errors list contains a single error object with the following structure:
       *
       * "errors" : [ {
       *     "msg" : "Some error text",
       *     "stackTrace" : "java.lang.RuntimeException: Some error text\r\n\tat com.quartzdesk.web.extjs.scheduler.quartz.QuartzJobAction.onList(QuartzJobAction.java:536).. \r\n\tat java.lang.Thread.run(Thread.java:748)\r\n"
       * } ]
       */
      Ext.widget( 'ExceptionErrorWindow', {
        title : errorTitle,
        qdesk : {
          errorText : responseJson.errors[ 0 ].msg,
          exceptionText : responseJson.errors[ 0 ].stackTrace,
          callback : callback
        }
      } ).show();

      return;
    }

    // format of validation error response is different from the format of other errors
    if ( errorType === QDesk.model.gui.ServerErrorType.VALIDATION )
    {
      // result.errors is an abject with fieldName : message pairs
      var msgs = Ext.Object.getValues( responseJson.errors );
      Ext.each( msgs, function ( msg, index )
      {
        errorMessages += msg;
        if ( index < ( msgs.length - 1 ) )
          errorMessages += '<p/>';
      } );
    }
    else if ( errorType === QDesk.model.gui.ServerErrorType.AUTHENTICATION )
    {
      errorMessages = QDesk.Msg.get( 'QDesk.userNotAuthenticatedMsg' );
    }
    else if ( errorType === QDesk.model.gui.ServerErrorType.AUTHORIZATION )
    {
      // result.errors is a list of objects, each object has a msg, exceptionType and requiredPermissions properties
      Ext.each( responseJson.errors, function ( error )
      {
        var exceptionType = error.exceptionType;
        if ( exceptionType === 'com.quartzdesk.domain.security.RequiresAnyPermissionsException' )
        {
          errorMessages = QDesk.Msg.get( 'QDesk.userNotAuthorizedAnyPermissionsRequiredMsg',
              QDesk.util.FormatUtils.htmlFormatPermissions( error.requiredPermissions ) );
        }
        else if ( exceptionType === 'com.quartzdesk.domain.security.RequiresAllPermissionsException' )
        {
          errorMessages = QDesk.Msg.get( 'QDesk.userNotAuthorizedAllPermissionsRequiredMsg',
              QDesk.util.FormatUtils.htmlFormatPermissions( error.requiredPermissions ) );
        }
        else
        {
          // unrecognized exceptionType - use a default error message
          QDesk.Log.warn( 'Unrecognized exception type in authorization error. Exception type: ' + exceptionType );
          errorMessages = QDesk.Msg.get( 'QDesk.userNotAuthorizedMsg' );
        }
      } );
    }
    else
    {
      // result.errors is a list of objects, each object has a msg property with the error message
      Ext.each( responseJson.errors, function ( error, index )
      {
        errorMessages += error.msg;
        if ( index < ( responseJson.errors.length - 1 ) )
          errorMessages += '<p/>';
      } );
    }

    Ext.MessageBox.show( {
      title : errorTitle,
      msg : errorMessages,
      buttons : Ext.MessageBox.OK,
      icon : Ext.MessageBox.ERROR,
      fn : callback
    } );
  },


  /**
   * Shows a progress dialog with the specified message. If a callback function
   * is provided, then the function is invoked when the progress dialog is closed.
   * @param msg a message.
   * @param callback an optional call-back function that will be invoked when
   * the dialog is closed.
   */
  showProgressDialog : function ( msg, callback )
  {
    Ext.MessageBox.show( {
      msg : msg,
      progress : true,
      closable : false,
      fn : Ext.isFunction( callback ) ? callback : undefined
    } );
  },


  /**
   * Shows a wait dialog with the specified message. If a callback function
   * is provided, then the function is invoked when the wait dialog is closed.
   * @param msg a message.
   * @param callback an optional call-back function that will be invoked when
   * the dialog is closed.
   */
  showWaitDialog : function ( msg, callback )
  {
    Ext.MessageBox.show( {
      msg : msg,
      wait : true,
//      waitConfig : {interval : 200},
      closable : false,
      fn : Ext.isFunction( callback ) ? callback : undefined
    } );
  },


  /**
   * Shows an info dialog with the specified message. If a callback function
   * is provided, then the function is invoked when the user closes the dialog
   * (either by clicking on the OK button, or by closing it by other means).
   * @param msg a message.
   * @param callback an optional call-back function that will be invoked when
   * the user closes the dialog.
   */
  showInfoDialog : function ( msg, callback )
  {
    Ext.MessageBox.show( {
      title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleInfo' ),
      msg : msg,
      buttons : Ext.MessageBox.OK,
      icon : Ext.MessageBox.INFO,
      fn : Ext.isFunction( callback ) ? callback : undefined
    } );
  },


  /**
   * Shows a warning dialog with the specified message. If a callback function
   * is provided, then the function is invoked when the user closes the dialog
   * (either by clicking on the OK button, or by closing it by other means).
   * @param msg a message.
   * @param callback an optional call-back function that will be invoked when
   * the user closes the dialog.
   */
  showWarningDialog : function ( msg, callback )
  {
    Ext.MessageBox.show( {
      title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleWarning' ),
      msg : msg,
      buttons : Ext.MessageBox.OK,
      icon : Ext.MessageBox.WARNING,
      fn : Ext.isFunction( callback ) ? callback : undefined
    } );
  },


  /**
   * Shows an error dialog with the specified message. If a callback function
   * is provided, then the function is invoked when the user closes the dialog
   * (either by clicking on the OK button, or by closing it by other means).
   * @param msg a message.
   * @param callback an optional call-back function that will be invoked when
   * the user closes the dialog.
   */
  showErrorDialog : function ( msg, callback )
  {
    Ext.MessageBox.show( {
      title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleError' ),
      msg : msg,
      buttons : Ext.MessageBox.OK,
      icon : Ext.MessageBox.ERROR,
      fn : Ext.isFunction( callback ) ? callback : undefined
    } );
  },


  /**
   * Shows an confirmation dialog with the specified message and Yes/No buttons.
   * If a callback function is provided, then the function is invoked when the
   * user closes the dialog (either by clicking on the Yes/No button, or by closing
   * it by other means).
   * @param msg a message.
   * @param callback an optional call-back function that will be invoked when
   * the user closes the dialog.
   */
  showConfirmDialog : function ( msg, callback )
  {
    Ext.MessageBox.show( {
      title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleConfirm' ),
      msg : msg,
      buttons : Ext.MessageBox.YESNO,
      icon : Ext.MessageBox.QUESTION,
      fn : Ext.isFunction( callback ) ? callback : undefined
    } );
  },


  /**
   * Puts a mask across the entire page body with an optional
   * message.
   * @param msg a message.
   * @see #maskEl
   */
  maskBody : function ( msg )
  {
    this.maskEl( Ext.getBody(), msg );
  },


  /**
   * Removes the mask from the page body.
   * @see #unmaskEl
   */
  unmaskBody : function ()
  {
    this.unmaskEl( Ext.getBody() );
  },


  /**
   * Puts a mask across the specified element with an optional
   * message.
   * @param element an ExtJS element.
   * @param msg a message.
   */
  maskEl : function ( element, msg )
  {
    element.mask( msg );
  },


  /**
   * Removes the mask from the specified ExtJS element.
   * @param element an ExtJS element.
   */
  unmaskEl : function ( element )
  {
    element.unmask();
  },


  /**
   * Depending on the {@link QDesk.ApplicationConfig} {@code infoBubblesEnabled} configuration option, this
   * method either opens a modal info dialog with the specified message and an OK button,
   * or a non-modal info bubble that automatically disappears after the configured timeout.
   * @param msg
   * @param callback
   */
  showInfo : function ( msg, callback )
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'infoBubblesEnabled' ) )
    {
      me.showInfoBubble( msg );
      if ( Ext.isFunction( callback ) )
      {
        callback.apply( this );
      }
    }
    else
    {
      me.showInfoDialog.apply( this, arguments );
    }
  },


  /**
   * Shows a non-modal info message bubble at the top of the viewport. The info
   * bubble automatically disappears after the configured {@link QDesk.ApplicationConfig}
   * {@code infoBubblesTimeout} delay.
   * @param msg a message.
   */
  showInfoBubble : function ( msg )
  {
    var me = this;

    var msgBox = Ext.get( 'msgDiv' );

    if ( !msgBox )
    {
      msgBox = Ext.DomHelper.insertFirst( document.body, { id : 'msgDiv' }, true );
    }

    var msgBoxBody = Ext.DomHelper.append( msgBox,
        '<div class="msg"><h3>' +
        QDesk.Msg.get( 'QDesk.util.UxUtils.messageTitleInfo' ) +
        '</h3><p>' + msg + '</p></div>',
        true ).hide();

    msgBoxBody = msgBoxBody.slideIn( 't' );
    msgBoxBody.ghost( 't', {
      delay : QDesk.ApplicationConfig.get( 'infoBubblesTimeout' ) * 1000,
      remove : true
    } );

    // message box must be shown on top of everything
    msgBox.setStyle( 'z-index', '999999' );
  },


  /**
   * Sets focus on the specified field.
   * @param field a field to set the focus on.
   * @param select flag indicating if the field's content should be selected.
   */
  focus : function ( field, select )
  {
    Ext.defer( function ()
    {
      field.focus( select ? true : false );
    }, QDesk.util.constant.Const.DELAY_FOCUS );
  },


  _showResponseError : function ( status, statusText, callback )
  {
    var errorText = QDesk.Msg.get( 'QDesk.util.UxUtils.operationErrorUnexpectedMsg' );

    if ( status == -1 )
    {
      if ( statusText == 'transaction aborted' )
      {
        errorText = QDesk.Msg.get( 'QDesk.util.UxUtils.operationErrorTransactionAbortedMsg' );
      }
    }
    else if ( status == 0 )
    {
      if ( statusText == '' )
      {
        // connection refused (server is down)
        errorText = QDesk.Msg.get( 'QDesk.util.UxUtils.operationErrorConnectionRefusedMsg' );
      }
      else if ( statusText == 'communication failure' )
      {
        // ajax timeout
        errorText = QDesk.Msg.get( 'QDesk.util.UxUtils.operationErrorRequestTimeoutMsg' );
      }
    }

    QDesk.util.UxUtils.showErrorDialog( errorText, callback );
  }
} );
