/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * A widget that allows a user to enter a phone number.
 * Inspired by Ext.ux.form.field.DateTime.
 */
Ext.define( 'QDesk.view.widget.WDateTimePeriod', {
  extend : 'Ext.form.FieldContainer',

  alias : 'widget.WDateTimePeriod',

  requires : [
    'QDesk.Msg',

    'Ext.ux.form.field.DateTime'
  ],

  mixins : {
    field : 'Ext.form.field.Field'
  },

  //configurables
  combineErrors : false,

  layout : 'hbox',
  readOnly : false,
  allowBlank : false,

  /**
   * @cfg {String} readOnlyCls
   * The CSS class applied to the component's main element when it is {@link #readOnly}.
   */
  readOnlyCls : Ext.baseCSSPrefix + 'form-readonly',

  /*
   * Important to set to '' to prevent ExtJS from applying x-item-disabled (opacity : 0.3) to this component when it
   * gets disabled. We want the x-item-disabled class to be applied only to the lowest-level components (individual
   * date and time fields), not to this upper-level container.
   */
  disabledCls : '',

  /*
   * START OF CONFIG PARAMETERS.
   */

  /**
   * The initial value.
   */
  value : null,

  /*
   * END OF CONFIG PARAMETERS.
   */

  /**
   * @private
   * @type Ext.ux.form.field.DateTime
   */
  _fromDateTime : null,

  /**
   * @private
   * @type Ext.ux.form.field.DateTime
   */
  _toDateTime : null,

  /**
   * @private
   * @type time period object
   */
  _oldValue : null,


  initComponent : function ()
  {
    var me = this;

    Ext.apply( me, {
      isFormField : true, //so it will be included in form field query's
      items : [
        /* start date time */
        {
          xtype : 'xdatetime',
          itemId : 'fromDateTime',
          allowBlank : false,
          timeFormat : QDesk.util.constant.DateFormat.timeFormatMedium,
          padding : '0 0 0 3',
          dateConfig : {
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldLabelFromTime' ),
            fieldTooltip : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldTooltipFromTime' ),
            emptyText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldEmptyTxtFromTimeDate' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldFormatTxtFromTimeDate' ),
            labelWidth : 50,
            width : 150
          },
          timeConfig : {
            emptyText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldEmptyTxtFromTimeTime' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldFormatTxtFromTimeTime' ),
            increment : 30, /* 30 minutes */
            width : 105
          },
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          }
        },

        /* end date time */
        {
          xtype : 'xdatetime',
          itemId : 'toDateTime',
          allowBlank : false,
          timeFormat : QDesk.util.constant.DateFormat.timeFormatMedium,
          padding : '0 5 0 5',
          dateConfig : {
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldLabelToTime' ),
            emptyText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldEmptyTxtToTimeDate' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldFormatTxtToTimeDate' ),
            labelWidth : 20,
            width : 120
          },
          timeConfig : {
            emptyText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldEmptyTxtToTimeTime' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.widget.WDateTimePeriod.fieldFormatTxtToTimeTime' ),
            increment : 30, /* 30 minutes */
            width : 105
          },
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          }
        }
      ]
    } );

    me.callParent();

    me._fromDateTime = me.down( '#fromDateTime' );
    me._toDateTime = me.down( '#toDateTime' );

    me.initField();

    // if the configured value is not specified or is null, use the current date time period
    if ( me.value == null )
    {
      /*
       * Set initial time period values.
       */
      var now = new Date();

      var todayMidnight = new Date();
      todayMidnight.setTime( now.getTime() );
      todayMidnight.setHours( 0, 0, 0, 0 );

      var tomorrowMidnight = new Date();
      tomorrowMidnight.setTime( now.getTime() );
      tomorrowMidnight.setDate( tomorrowMidnight.getDate() + 1 );
      tomorrowMidnight.setHours( 0, 0, 0, 0 );

      me.setValue( {
        from : todayMidnight,
        to : tomorrowMidnight
      } );
    }

    // force validation before the widget gets displayed
    me.on( 'beforerender', function ()
    {
      me.isValid();
    } );
  },

  // // private
  // beforeDestroy : function ()
  // {
  //   Ext.destroy( this.fieldCt );
  //   this.callParent( arguments );
  // },

  // private
  delegateFn : function ( fn )
  {
    this.items.each( function ( item )
    {
      if ( item[ fn ] )
      {
        item[ fn ]();
      }
    } );
  },


  // @inheritdoc
  getErrors : function ()
  {
    var errors = [];

    errors.concat( this._fromDateTime.getErrors() );
    errors.concat( this._toDateTime.getErrors() );

    return errors;
  },


  // @inheritdoc
  isValid : function ()
  {
    this.setValidation( null );

    if ( this._fromDateTime.isValid() && this._toDateTime.isValid() )
    {
      // check that FROM <= TO
      var fromValue = this._fromDateTime.getValue();
      var toValue = this._toDateTime.getValue();

      if ( fromValue != null && toValue != null )
      {
        if ( fromValue.getTime() <= toValue.getTime() )
        {
          // calling isValid revalidates the fields and updates the error highlighting
          this._fromDateTime.isValid();
          this._toDateTime.isValid();

          return true;
        }
        else
        {
          this.setValidation( QDesk.Msg.get( 'QDesk.view.widget.WDateTimePeriod.errorFromGreaterThanTo' ) );
        }
      }
    }

    // calling isValid revalidates the fields and updates the error highlighting
    this._fromDateTime.isValid();
    this._toDateTime.isValid();

    return false;
  },


  // @inheritdoc
  getSubmitValue : function ()
  {
    return this.getValue();
  },


  /**
   * Returns the value of this date-time period field as an object with the following structure:
   * <pre>
   * {
   *   from : Date
   *   to   : Date
   * }
   * </pre>
   * @return {Object}
   */
  getValue : function ()
  {
    if ( this.isValid() )
    {
      var from = this._fromDateTime.getValue();
      var to = this._toDateTime.getValue();

      /*
       * if fromDateTime and toDateTime allow blank values, they are valid even if their values are null
       */
      if ( from != null && to != null )
      {
        return {
          from : from,
          to : to
        };
      }
    }

    return null;
  },


  // @inheritdoc
  setValue : function ( value )
  {
    if ( value )
    {
      this._fromDateTime.setValue( value.from );
      this._toDateTime.setValue( value.to );
    }
    else
    {
      this._fromDateTime.setValue( null );
      this._toDateTime.setValue( null );
    }

    // validate this field and update the error highlighting
    this.isValid();
  },


  // @inheritdoc
  isDirty : function ()
  {
    return this._fromDateTime.isDirty()
        || this._toDateTime.isDirty();
  },


  // @inheritdoc
  reset : function ()
  {
    this.delegateFn( 'reset' );
  },


  // @inheritdoc
  resetOriginalValue : function ()
  {
    this._fromDateTime.resetOriginalValue();
    this._toDateTime.resetOriginalValue();
  },


  // @inheritdoc
  markInvalid : function ( errors )
  {
    this._fromDateTime.markInvalid( errors );
    this._toDateTime.markInvalid( errors );
  },


  // @inheritdoc
  clearInvalid : function ( errors )
  {
    this._fromDateTime.clearInvalid( errors );
    this._toDateTime.clearInvalid( errors );
  },


  // @inheritdoc
  setReadOnly : function ( readOnly )
  {
    this._fromDateTime.setReadOnly( readOnly );
    this._toDateTime.setReadOnly( readOnly );

    this[ readOnly ? 'addCls' : 'removeCls' ]( this.readOnlyCls );
    this.readOnly = readOnly;
  },


  // @inheritdoc
  setValidation : function ( error )
  {
    this._fromDateTime.setValidation( error );
    this._toDateTime.setValidation( error );
  },


  // private
  onDisable : function ()
  {
    this.delegateFn( 'disable' );
  },


  // private
  onEnable : function ()
  {
    this.delegateFn( 'enable' );
  },


  // private
  onFieldChange : function ()
  {
    this.isValid();

    var newValue = this.getValue();
    this.fireEvent( 'change', this, newValue, this._oldValue );
    this._oldValue = newValue;
  }
} );