/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * A widget that allows a user to enter a phone number.
 * Inspired by Ext.ux.form.field.DateTime.
 */
Ext.define( 'QDesk.view.widget.WPhoneNumber', {
  extend : 'Ext.form.FieldContainer',

  alias : 'widget.WPhoneNumber',

  requires : [],

  mixins : {
    field : 'Ext.form.field.Field'
  },

  //configurables
  combineErrors : true,
  layout : 'hbox',
  readOnly : false,
  allowBlank : true,

  defaults : {
    hideLabel : true
  },

  /*
   * Initial value.
   */
  value : null,

  /**
   * @property countryCodeField
   * @type Ext.form.field.ComboBox
   */
  countryCodeField : null,

  /**
   * @property nationalDestCodeField
   * @type Ext.form.field.Text
   */
  nationalDestCodeField : null,

  /**
   * @property subscriberNumberField
   * @type Ext.form.field.Text
   */
  subscriberNumberField : null,

  /**
   * @cfg {String} readOnlyCls
   * The CSS class applied to the component's main element when it is {@link #readOnly}.
   */
  readOnlyCls : Ext.baseCSSPrefix + 'form-readonly',

  /*
   * Important to set to '' to prevent ExtJS from applying x-item-disabled (opacity : 0.3) to this component when it
   * gets disabled. We want the x-item-disabled class to be applied only to the lowest-level components (individual
   * text fields), not to this upper-level container.
   */
  disabledCls : '',


  initComponent : function ()
  {
    var me = this;

    Ext.apply( me, {
      isFormField : true, //so it will be included in form field query's
      items : [
        {
          xtype : 'displayfield',
          value : '+'
        },
        {
          xtype : 'textfield',
          itemId : 'countryCode',
          isFormField : false,  //exclude from field query's
          submitValue : false,  // prevent the field from being submitted even when it is not disabled.
          // validateOnChange : false,
          allowBlank : me.allowBlank,
          regex : /^[0-9]{1,3}$/,
          width : 40,
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          }
        },
        {
          xtype : 'displayfield',
          value : '',
          margin : '0 5 0 5'
        },

        {
          xtype : 'textfield',
          itemId : 'nationalDestCode',
          isFormField : false,  //exclude from field query's
          submitValue : false,  // prevent the field from being submitted even when it is not disabled.
          // validateOnChange : false,
          allowBlank : true,
          regex : /^[0-9]{1,4}$/,
          width : 50,
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          }
        },

        {
          xtype : 'displayfield',
          value : ' - ',
          margin : '0 5 0 5'
        },

        {
          xtype : 'textfield',
          itemId : 'subscriberNumber',
          isFormField : false,  //exclude from field query's
          submitValue : false,  // prevent the field from being submitted even when it is not disabled.
          // validateOnChange : false,
          allowBlank : me.allowBlank,
          regex : /^[0-9\- ]{1,12}$/,
          width : 150,
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          }
        }
      ]
    } );

    me.callParent();

    me.countryCodeField = me.down( '#countryCode' );
    me.nationalDestCodeField = me.down( '#nationalDestCode' );
    me.subscriberNumberField = me.down( '#subscriberNumber' );

    me.initField();
  },

  // // private
  // beforeDestroy : function ()
  // {
  //   Ext.destroy( this.fieldCt );
  //   this.callParent( arguments );
  // },

  // private
  delegateFn : function ( fn )
  {
    this.items.each( function ( item )
    {
      if ( item[ fn ] )
      {
        item[ fn ]();
      }
    } );
  },

  // @inheritdoc
  getErrors : function ()
  {
    return [].concat( this.countryCodeField.getErrors() )
        .concat( this.nationalDestCodeField.getErrors() )
        .concat( this.subscriberNumberField.getErrors() );
  },

  getSubmitValue : function ()
  {
    var countryCodeValue = this.countryCodeField.getValue();
    var nationalDestCodeValue = this.nationalDestCodeField.getValue();
    var subscriberNumberValue = this.subscriberNumberField.getValue();

    if ( countryCodeValue && subscriberNumberValue )
    {
      return '(' + countryCodeValue + ')' + ( nationalDestCodeValue ? nationalDestCodeValue : '' ) + '-' +
          subscriberNumberValue;
    }
    else
    {
      return null;
    }
  },

  /**
   * Returns the value of this phone number as an object with the following structure:
   * <pre>
   * {
   *   countryCode : String       // required
   *   nationalDestCode : String  // optional
   *   subscriberNumber : String  // required
   * }
   * </pre>
   * @return {Object}
   */
  getValue : function ()
  {
    var countryCodeValue = this.countryCodeField.getValue();
    var nationalDestCodeValue = this.nationalDestCodeField.getValue();
    var subscriberNumberValue = this.subscriberNumberField.getValue();

    if ( countryCodeValue && subscriberNumberValue )
    {
      var result = {
        countryCode : countryCodeValue
      };

      if ( nationalDestCodeValue )
      {
        Ext.apply( result, {
          nationalDestCode : nationalDestCodeValue
        } );
      }

      Ext.apply( result, {
        subscriberNumber : subscriberNumberValue
      } );

      return result;
    }
    else
    {
      return null;
    }
  },

  // @inheritdoc
  isDirty : function ()
  {
    var dirty = false;
    if ( this.rendered && !this.disabled )
    {
      this.items.each( function ( item )
      {
        if ( item.isDirty() )
        {
          dirty = true;
          return false;
        }
      } );
    }
    return dirty;
  },

  // private
  onDisable : function ()
  {
    this.delegateFn( 'disable' );
  },

  // private
  onEnable : function ()
  {
    this.delegateFn( 'enable' );
  },

  // private
  onFieldChange : function ()
  {
    this.fireEvent( 'change', this, this.getValue() );
  },

  // @inheritdoc
  reset : function ()
  {
    this.delegateFn( 'reset' );
  },

  // @inheritdoc
  resetOriginalValue : function ()
  {
    this.countryCodeField.resetOriginalValue();
    this.nationalDestCodeField.resetOriginalValue();
    this.subscriberNumberField.resetOriginalValue();
  },

  markInvalid : function ( errors )
  {
    this.countryCodeField.markInvalid( errors );
    this.nationalDestCodeField.markInvalid( errors );
    this.subscriberNumberField.markInvalid( errors );
  },

  clearInvalid : function ( errors )
  {
    this.countryCodeField.clearInvalid( errors );
    this.nationalDestCodeField.clearInvalid( errors );
    this.subscriberNumberField.clearInvalid( errors );
  },

  setReadOnly : function ( readOnly )
  {
    this.countryCodeField.setReadOnly( readOnly );
    this.nationalDestCodeField.setReadOnly( readOnly );
    this.subscriberNumberField.setReadOnly( readOnly );

    this[ readOnly ? 'addCls' : 'removeCls' ]( this.readOnlyCls );
    this.readOnly = readOnly;
  },

  setValue : function ( value )
  {
    if ( value )
    {
      this.countryCodeField.setValue( value.countryCode );
      this.nationalDestCodeField.setValue( value.nationalDestCode );
      this.subscriberNumberField.setValue( value.subscriberNumber );
    }
    else
    {
      this.countryCodeField.setValue( null );
      this.nationalDestCodeField.setValue( null );
      this.subscriberNumberField.setValue( null );
    }
  },

  isValid : function ()
  {
    return this.countryCodeField.isValid() &&
        this.nationalDestCodeField.isValid() &&
        this.subscriberNumberField.isValid();
  }
} );