/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * A widget that allows a user to enter an expression, e.g. a JavaScript expression.
 */
Ext.define( 'QDesk.view.widget.expression.WExpressionEditor', {
  extend : 'Ext.form.FieldContainer',

  alias : 'widget.expression.WExpressionEditor',

  requires : [
    'QDesk.Msg',

    'QDesk.model.common.ExpressionLanguage',

    'QDesk.store.enums.ExpressionLanguageStore',

    'QDesk.util.FormUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.widget.expression.JSScriptingObjectsWindow',

    'Ext.ux.form.IconCombo'
  ],

  mixins : {
    field : 'Ext.form.field.Field'
  },

  //configurables
  combineErrors : false,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  cls : 'WExpressionEditor',
  readOnly : false,
  allowBlank : false,

  /**
   * @cfg {String} readOnlyCls
   * The CSS class applied to the component's main element when it is {@link #readOnly}.
   */
  readOnlyCls : Ext.baseCSSPrefix + 'form-readonly',

  /*
   * START OF CONFIG PARAMETERS.
   */

  /**
   * The initial value.
   */
  value : null,

  /**
   * The expression type. See QDesk.model.common.ExpressionType.
   */
  expressionType : null,

  /**
   * Tooltip for the expression language combo.
   */
  languageTooltip : null,

  /**
   * The empty value text shown in the expression text area.
   */
  textEmptyText : null,

  /**
   * Tooltip for the expression text area.
   */
  textTooltip : null,

  /**
   * Tooltip of the Show Expression Objects button.
   */
  buttonShowExpressionObjectsTooltip : null,

  /**
   * Tooltip of the Validate Expression button.
   */
  buttonValidateExpressionTooltip : null,

  /**
   * Required config parameter with the objects tree store.
   */
  objectsStore : null,

  /**
   * Required config parameter with the data types tree store.
   */
  dataTypesStore : null,

  /**
   * An optional config parameter pointing to a function returning extra parameters
   * passed to the validation action.
   */
  extraValidateParamFn : null,

  /*
   * END OF CONFIG PARAMETERS.
   */

  /**
   * @private
   * @type Ext.ux.form.IconCombo
   */
  _languageCombo : null,

  /**
   * @private
   * @type Ext.form.field.TextArea
   */
  _textArea : null,

  /**
   * @private
   * @type Ext.form.field.Button
   */
  _buttonShowExpressionObjects : null,

  /**
   * @private
   * @type Ext.form.field.Button
   */
  _buttonValidateExpression : null,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.expressionType )
      Ext.Error.raise( 'Required config parameter missing: expressionType' );

    if ( !config.objectsStore )
      Ext.Error.raise( 'Required config parameter missing: objectsStore' );

    if ( !config.dataTypesStore )
      Ext.Error.raise( 'Required config parameter missing: dataTypesStore' );

    if ( Ext.isEmpty( config.textEmptyText ) )
    {
      Ext.apply( config, {
        textEmptyText : QDesk.Msg.get( 'QDesk.view.widget.expression.WExpressionEditor.emptyValueText' )
      } )
    }

    // language combo tooltip
    if ( Ext.isEmpty( config.languageTooltip ) )
    {
      Ext.apply( config, {
        languageTooltip : QDesk.Msg.get(
            'QDesk.view.widget.expression.WExpressionEditor.fieldTooltipLanguage' )
      } )
    }

    // expression text tooltip
    if ( Ext.isEmpty( config.textTooltip ) )
    {
      Ext.apply( config, {
        textTooltip : QDesk.Msg.get(
            'QDesk.view.widget.expression.WExpressionEditor.fieldTooltipText' )
      } )
    }

    // START OF BUTTON TOOLTIPS
    if ( Ext.isEmpty( config.buttonShowExpressionObjectsTooltip ) )
    {
      Ext.apply( config, {
        buttonShowExpressionObjectsTooltip : QDesk.Msg.get(
            'QDesk.view.widget.expression.WExpressionEditor.buttonShowExpressionObjectsTooltip' )
      } )
    }

    if ( Ext.isEmpty( config.buttonValidateExpressionTooltip ) )
    {
      Ext.apply( config, {
        buttonValidateExpressionTooltip : QDesk.Msg.get(
            'QDesk.view.widget.expression.WExpressionEditor.buttonValidateExpressionTooltip' )
      } )
    }
    // END OF BUTTON TOOLTIPS

    Ext.apply( this, config );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    Ext.apply( me, {
      isFormField : true, //so it will be included in form field query's
      items : [
        {
          xtype : 'iconcombo',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.widget.expression.WExpressionEditor.fieldLabelLanguage' ),
          fieldTooltip : me.languageTooltip,
          labelAlign : 'left',
          labelWidth : me.labelWidth,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          value : me.value ? me.value.language : null,
          store : 'enums.ExpressionLanguageStore',
          valueField : 'value',
          displayField : 'valueLabel',
          iconClsField : 'value',
          allowBlank : me.allowBlank,
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          }
        },
        {
          xtype : 'container',
          layout : 'column',
          items : [
            {
              xtype : 'textareafield',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.widget.expression.WExpressionEditor.fieldLabelText' ),
              fieldTooltip : me.textTooltip,
              emptyText : me.textEmptyText,
              labelAlign : 'top',
              value : me.value ? me.value.expression : null,
              columnWidth : 1,
              height : 200,
              allowBlank : me.allowBlank,
              listeners : {
                change : function ()
                {
                  me.onFieldChange();
                },
                scope : me
              }
            },
            {
              xtype : 'container',
              border : false,
              layout : 'vbox',
              align : 'left',
              style : {
                marginLeft : '3px',
                marginTop : '22px'
              },
              items : [
                {
                  xtype : 'button',
                  itemId : 'buttonShowExpressionObjects',
                  tooltip : me.buttonShowExpressionObjectsTooltip,
                  iconCls : 'expressionShowObjects',
                  listeners : {
                    click : function ()
                    {
                      me._showExpressionObjects( me._languageCombo.getValue() );
                    }
                  }
                },
                {
                  xtype : 'button',
                  itemId : 'buttonValidateExpression',
                  tooltip : me.buttonValidateExpressionTooltip,
                  iconCls : 'expressionValidate',
                  margin : '3 0 0 0',
                  listeners : {
                    click : function ()
                    {
                      me._validateExpression( me.getValue() );
                    }
                  }
                }
              ]
            }
          ]
        }
      ]
    } );

    me.callParent();

    me._languageCombo = me.down( 'iconcombo' );
    me._textArea = me.down( 'textareafield' );

    me._buttonShowExpressionObjects = me.down( '#buttonShowExpressionObjects' );
    me._buttonValidateExpression = me.down( '#buttonValidateExpression' );

    // set the initial button states
    me._updateButtonsStates();

    me.initField();
  },


  // private
  delegateFn : function ( fn )
  {
    this.items.each( function ( item )
    {
      if ( item[ fn ] )
      {
        item[ fn ]();
      }
    } );
  },


  // @inheritdoc
  // getErrors : function ()
  // {
  //   return [].concat( this._languageCombo.getErrors() )
  //       .concat( this._textArea.getErrors() );
  // },


  getSubmitValue : function ()
  {
    return this.getValue();
  },


  /**
   * Returns the value of this expression as an object with the following structure:
   * <pre>
   * {
   *   language : QDesk.model.common.ExpressionLanguage  // required
   *   text     : String                                 // required
   * }
   * </pre>
   * @return {Object}
   */
  getValue : function ()
  {
    var language = this._languageCombo.getValue();
    var text = this._textArea.getValue();

    if ( language && text )
    {
      return {
        language : language,
        text : text
      };
    }
    else
    {
      return null;
    }
  },


  // @inheritdoc
  isDirty : function ()
  {
    return this._languageCombo.isDirty()
        || this._textArea.isDirty();
  },


  // private
  onDisable : function ()
  {
    this.delegateFn( 'disable' );
  },


  // private
  onEnable : function ()
  {
    this.delegateFn( 'enable' );
  },


  // private
  onFieldChange : function ()
  {
    var me = this;

    me._updateButtonsStates();
    me.fireEvent( 'change', me, me.getValue() );
  },


  // @inheritdoc
  reset : function ()
  {
    this._languageCombo.reset();
    this._textArea.reset();
  },


  // @inheritdoc
  resetOriginalValue : function ()
  {
    this._languageCombo.resetOriginalValue();
    this._textArea.resetOriginalValue();
  },


  markInvalid : function ( errors )
  {
    this._textArea.markInvalid( errors );
  },


  clearInvalid : function ( errors )
  {
    this._textArea.clearInvalid( errors );
  },


  setReadOnly : function ( readOnly )
  {
    this._languageCombo.setReadOnly( readOnly );
    this._textArea.setReadOnly( readOnly );

    this[ readOnly ? 'addCls' : 'removeCls' ]( this.readOnlyCls );
    this.readOnly = readOnly;
  },


  setValue : function ( value )
  {
    if ( value )
    {
      this._languageCombo.setValue( value.language );
      this._textArea.setValue( value.text );
    }
    else
    {
      this._languageCombo.setValue( null );
      this._textArea.setValue( null );
    }
  },


  isValid : function ()
  {
    return this._languageCombo.isValid()
        && this._textArea.isValid();
  },


  /**
   * Enables/disables buttons based on the current language combo and expression text area values.
   * @private
   */
  _updateButtonsStates : function ()
  {
    var me = this;

    if ( me._languageCombo.getValue() == null )
    {
      // no language specified = all buttons should be disabled
      me._buttonShowExpressionObjects.disable();
      me._buttonValidateExpression.disable();
    }
    else
    {
      me._buttonShowExpressionObjects.enable();

      if ( Ext.isEmpty( me._textArea.getValue() ) )
      {
        // no expression text = disable the expression validation button
        me._buttonValidateExpression.disable();
      }
      else
      {
        // non-empty expression text = enable the expression validation button
        me._buttonValidateExpression.enable();
      }
    }
  },


  /**
   * Shows the scripting objects window.
   * @param language a language.
   * @private
   */
  _showExpressionObjects : function ( language )
  {
    var me = this;

    var window;
    switch ( language )
    {
      case QDesk.model.common.ExpressionLanguage.JAVASCRIPT:
        window = Ext.widget( 'expression.JSScriptingObjectsWindow', {
          objectsStore : me.objectsStore,
          dataTypesStore : me.dataTypesStore
        } );
        break;

      default:
        Ext.Error.raise( 'No scripting objects available for language: ' + selectedExprLang );
    }

    if ( window )
    {
      window.on( 'valueSelected',
          function ( value )
          {
            QDesk.util.FormUtils.insertTextAtCursor( me._textArea, value );
          } );

      window.show();
    }
  },


  /**
   * Validates the specified expression value.
   * @param value a value.
   * @private
   */
  _validateExpression : function ( value )
  {
    var me = this;

    if ( Ext.isEmpty( value ) || Ext.isEmpty( value.text ) )
      return;

    var maskEl = me.up( 'window' ).getEl();

    QDesk.util.UxUtils.maskEl( maskEl,
        QDesk.Msg.get( 'QDesk.view.widget.expression.WExpressionEditor.maskValidateMsg' ) );

    var params = {
      type : me.expressionType,
      expression : Ext.encode( value )
    };

    if ( me.extraValidateParamFn && Ext.isFunction( me.extraValidateParamFn ) )
    {
      Ext.apply( params, me.extraValidateParamFn.call( me ) );
    }

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.expressionActionValidate,
      params : params,
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskEl( maskEl );

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfoDialog(
                QDesk.Msg.get( 'QDesk.view.widget.expression.WExpressionEditor.successValidateMsg' ) );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }
      }
    } );
  }
} );