/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.navigation.NavigationPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.ViewUrl',

    'QDesk.model.customization.gui.navigation.Node',
    'QDesk.model.customization.gui.navigation.Tab',

    'QDesk.view.work.navigation.connections.ConnectionsTreePanel',
    'QDesk.view.work.navigation.settings.SettingsTreePanel'
  ],

  alias : 'widget.NavigationPanel',

  cls : 'navigationPanel',

  split : true,
  collapsible : true,

  layout : {
    type : 'accordion'
  },


  /*
   * Commented out, because with the introduction of custom tabs, the list of tabs is loaded
   * dynamically.
   */
  //items : [
  //  {
  //    xtype : 'ConnectionsTreePanel',
  //    itemId : 'connectionsTreePanel',
  //    //title : QDesk.Msg.get( 'QDesk.view.work.navigation.NavigationPanel.titleConnectionsPanel' ),
  //    collapsed : false
  //  },
  //  {
  //    xtype : 'SettingsTreePanel',
  //    itemId : 'settingsTreePanel',
  //    //title : QDesk.Msg.get( 'QDesk.view.work.navigation.NavigationPanel.titleSettingsPanel' ),
  //    collapsed : true
  //  }
  //],


  initComponent : function ()
  {
    var me = this;
    me.callParent();

    var tabStore = Ext.StoreManager.lookup( 'customization.gui.navigation.TabStore' );
    tabStore.load( {
      scope : this,
      callback : function ( records, operation, success )
      {
        Ext.each( records, function ( record )
        {
          var widget;
          var type = record.get( 'type' );
          switch ( type )
          {
            case 'XTypeTab':
              widget = me._createXTypeTab( record );
              break;

            case 'CustomTab':
              widget = me._createCustomTab( record );
              break;

            default:
              Ext.Error.raise( 'Unsupported tab type: ' + type );
          }

          me.add( widget );
        } );

        // this event triggers loading
        me.fireEvent( 'loaded' );
      }
    } );
  },


  /**
   * Creates a tab from the specified XTYPE tab definition record (QDesk.model.customization.gui.navigation.Tab
   * instance).
   * @param tabDefRecord a QDesk.model.customization.gui.navigation.Tab instance.
   * @returns the created tab.
   * @private
   */
  _createXTypeTab : function ( tabDefRecord )
  {
    var me = this;

    return Ext.widget( tabDefRecord.get( 'xtype' ), {
      itemId : tabDefRecord.get( 'itemId' ),
      title : me._getTabTitle( tabDefRecord ),
      collapsed : tabDefRecord.get( 'collapsed' )
    } );
  },


  /**
   * Creates a tab from the specified CUSTOM tab definition record (QDesk.model.customization.gui.navigation.Tab
   * instance).
   * @param tabDefRecord a QDesk.model.customization.gui.navigation.Tab instance.
   * @returns the created tab.
   * @private
   */
  _createCustomTab : function ( tabDefRecord )
  {
    var me = this;

    var store = Ext.create( 'Ext.data.TreeStore', {
      model : 'QDesk.model.customization.gui.navigation.Node',
      root : {
        expanded : true,
        children : tabDefRecord.get( 'children' )
      }
    } );

    var panel = Ext.widget( 'treepanel', {
      itemId : tabDefRecord.get( 'itemId' ),
      title : me._getTabTitle( tabDefRecord ),
      cls : 'customTab',
      collapsed : tabDefRecord.get( 'collapsed' ),

      displayField : 'title',

      useArrows : true,
      rootVisible : false,
      store : store
    } );

    panel.on( 'itemclick', function ( treepanel, record, item, index, event )
    {
      var type = record.get( 'type' );
      if ( type == 'CustomLinkNode' )
      {
        // stop the propagation of the event and prevent the browser from opening the link
        event.stopEvent();
      }
    } );

    panel.on( 'itemdblclick', function ( treepanel, record, item, index, event )
    {
      var type = record.get( 'type' );
      if ( type == 'CustomLinkNode' )
      {
        // stop the propagation of the event and prevent the browser from opening the link
        event.stopEvent();

        var linkHref = record.get( 'href' );
        var linkHrefTarget = record.get( 'hrefTarget' );

        if ( Ext.isEmpty( linkHrefTarget ) ) // no target window specified (= open in a tab inside of the main panel)
        {
          var nodeId = record.get( 'id' );
          var linkTitle = record.get( 'title' );
          var linkIcon = record.get( 'icon' );

          // view the URL in a panel displayed in the Main Tab Panel
          Ext.create( 'QDesk.controller.command.ViewUrl', {
            id : nodeId,
            tabTitle : linkTitle,
            tabIcon : linkIcon, /* tab uses the same icon as the link */
            url : linkHref
          } ).execute();
        }
        else
        {
          window.open( linkHref, linkHrefTarget );
        }
      }
    } );

    return panel;
  },


  /**
   * Returns the title for the specified tab.
   * @param tabDefRecord a QDesk.model.customization.gui.navigation.Tab instance.
   * @returns the tab title.
   * @private
   */
  _getTabTitle : function ( tabDefRecord )
  {
    var title = tabDefRecord.get( 'title' );

    if ( title && title.indexOf( 'i18n:' ) == 0 )
    {
      var titleKey = title.substr( 'i18n:'.length );
      title = QDesk.Msg.get( titleKey );
    }

    return title;
  }
} );
