/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.FormatUtils',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid'
  ],

  alias : 'widget.QuartzMisfiredTriggersTab',

  iconCls : 'quartzMisfiredTriggersSubTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    var publicApiReady = schedulerInfo.isPublicApiReady();

    var items = [];

    items.push(
        {
          xtype : 'WarningMessagePanel',
          style : {
            margin : '5px'
          },
          hidden : true
        }
    );

    var fetchPeriod = QDesk.util.CommonUtils.getDefaultFetchPeriod( schedulerInfo.getSchedulerTimeZone(),
        QDesk.ApplicationConfig.get( 'quartz.schedulerMisfiredTriggersGrid.fetchPeriodLength' ) );

    items.push(
        {
          xtype : 'QuartzMisfiredTriggersGrid',
          itemId : 'misfiredTriggersGrid',
          flex : 1,
          hidden : true,
          qdesk : {
            connectionCtx : connectionCtx,
            gridId : 'quartz.schedulerMisfiredTriggersGrid',
            fetchPeriod : fetchPeriod,
            maxFetchSize : QDesk.ApplicationConfig.get( 'quartz.schedulerMisfiredTriggersGrid.maxFetchSize' )
          }
        }
    );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var warningVisible = false;
    var warningMsg = null;
    var gridVisible = false;

    if ( licensedFeatures.MisfiredTriggers )
    {
      if ( jvmAgentReady )
      {
        if ( jvmAgentInfo.isQuartzMisfiredTriggersTrackingEnabled() )
        {
          if ( publicApiReady )
          {
            if ( jvmAgentInfo.isQuartzMisfiredTriggersProcessingEnabled() )
            {
              // everything is OK
              gridVisible = true;
            }
            else
            {
              // misfired trigger processing is not enabled
              warningVisible = true;
              warningMsg = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.misfiredTriggersProcessingDisabledMsg' );

              // grid visible because misfired triggers are tracked (inserted), they are just not processed
              gridVisible = true;
            }
          }
          else
          {
            // public API is not ready
            warningVisible = true;
            warningMsg = me.getPublicApiWarningMessage( connectionCtx );
            gridVisible = true;
          }
        }
        else
        {
          // misfired trigger tracking is not enabled
          warningVisible = true;
          warningMsg = QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.misfiredTriggersTrackingDisabledMsg' );

          // grid hidden because tracking of misfired triggers is disabled in the agent
          gridVisible = false;
        }
      }
      else
      {
        warningVisible = true;
        warningMsg = me.getJvmAgentWarningMessage( connectionCtx );

        // grid hidden because agent is not ready
        gridVisible = false;
      }
    }
    else
    {
      warningVisible = true;
      warningMsg = QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
          QDesk.util.license.LicensedProduct.productName )
    }

    /*
     * Intentionally over-nested a bit so that we use identical layout to other tabs that
     * have a license restrictions related message panel above the grid.
     */
    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.tooltip' ),
      items : [
        {
          xtype : 'panel',
          border : false,
          layout : 'fit',
          flex : 1,
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'vbox',
                align : 'stretch'
              },
              items : items
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    if ( warningVisible )
    {
      var warningPanel = me.down( 'WarningMessagePanel' );
      warningPanel.setMessage( warningMsg );
      warningPanel.setVisible( true );
    }

    if ( gridVisible )
    {
      me.down( '#misfiredTriggersGrid' ).setVisible( true );
    }
  },


  /**
   * Overridden method from the base class that returns the warning message shown at the top of the exec history grid.
   * @param connectionCtx a {@link QDesk.ConnectionContext} instance.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.publicApiNotLoadedCorrectlyMsg' );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.publicApiIncompatibleVersionMsg',
            QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get(
          'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.publicApiNotDetectedMsg' );
    }
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab.title' );
    if ( !licensedFeatures.ExecutionHistory )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );