/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.groups.EditGroupForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.enums.security.PermissionStore',
    'QDesk.store.security.users.UserStore',

    'QDesk.util.FormUtils',
    'QDesk.util.security.GroupUtils',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.security.groups.EditGroupFormMode',

    'QDesk.view.widget.WPhoneNumber'
  ],

  alias : 'widget.EditGroupForm',

  bodyPadding : '5 10 10 10',

  border : false,

  scrollable : true,

  fieldDefaults : {
    labelWidth : 80
  },

  defaultButton : 'actionButton',

  qdesk : {
    /*
     * Mode of the form (see EditGroupFormMode).
     */
    mode : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    var userStore = Ext.create( 'QDesk.store.security.users.UserStore', {
      sorters : [
        {
          property : 'displayName',
          direction : 'ASC'
        }
      ]
    } );

    /*
     * Important to escape HTML in user displayName attribute that is shown in the user item selector field.
     * JMO: this is a bit of a hack as I have not found a way to make the item selector use a custom renderer
     * that would escape the displayed values.
     */
    userStore.on( 'load', function ( store, records, successful )
    {
      if ( successful )
      {
        Ext.each( records, function ( record )
        {
          record.set( 'displayName', Ext.String.htmlEncode( record.get( 'displayName' ) ) );
        } );
      }
    } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'WarningMessagePanel',
          itemId : 'warningMessagePanel',
          hidden : true,
          style : {
            marginBottom : '5px'
          }
        },

        {
          xtype : 'hiddenfield',
          itemId : 'id',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'system',
          name : 'system'
        },

        {
          xtype : 'textfield',
          itemId : 'name',
          name : 'name',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.security.groups.EditGroupForm.fieldLabelName' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.security.groups.EditGroupForm.fieldTooltipName' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.security.groups.EditGroupForm.fieldEmptyTxtName' ),
          allowBlank : false,
          maxLength : 128,
          maxWidth : 350
        },

        {
          xtype : 'tabpanel',
          padding : '0 0 10 0',
          items : [
            {
              xtype : 'panel',
              padding : '10 10 5 10',
              border : false,
              iconCls : 'permissionsTabIcon',
              title : QDesk.Msg.get(
                  'QDesk.view.work.security.groups.EditGroupForm.tabLabelPermissions' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.groups.EditGroupForm.tabTooltipPermissions' ),
              layout : 'fit',
              // defaults : {
              //   anchor : '100%'
              // },
              items : [
                {
                  xtype : 'itemselectorfield',
                  itemId : 'permissions',
                  name : 'permissionIds',
                  fromTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.groups.EditGroupForm.fieldTitlePermissionsFrom' ),
                  toTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.groups.EditGroupForm.fieldTitlePermissionsTo' ),
                  allowBlank : true,
                  displayField : 'valueLabelWithTooltip',
                  valueField : 'value',
                  //minSelections : 1,
                  buttons : [ 'add', 'remove' ],
                  store : 'enums.security.PermissionStore',
                  height : 400
                }
              ]
            },
            {
              xtype : 'panel',
              padding : '10 10 5 10',
              border : false,
              iconCls : 'membersTabIcon',
              title : QDesk.Msg.get(
                  'QDesk.view.work.security.groups.EditGroupForm.tabLabelUsers' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.groups.EditGroupForm.tabTooltipUsers' ),
              layout : 'fit',
              // defaults : {
              //   anchor : '100%'
              // },
              items : [
                {
                  xtype : 'itemselectorfield',
                  itemId : 'users',
                  name : 'userIds',
                  fromTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.groups.EditGroupForm.fieldTitleUsersFrom' ),
                  toTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.groups.EditGroupForm.fieldTitleUsersTo' ),
                  allowBlank : true,
                  displayField : 'displayName',
                  valueField : 'id',
                  //minSelections : 1,
                  buttons : [ 'add', 'remove' ],
                  store : userStore,
                  height : 400
                }
              ]
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true,
          qdRequiresAnyPermissions : me._getRequiredPermissions( config.qdesk.mode )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    me._setDefaultFormValues();
  },


  /**
   * Loads the specified group record ({@link Group} instance) into this form.
   * @param group the group record.
   */
  loadRecord : function ( group )
  {
    var me = this;
    me.callParent( arguments );

    var isSystemGroup = group.get( 'system' );
    if ( isSystemGroup )
    {
      var warningMsgPanel = me.down( '#warningMessagePanel' );
      warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.work.security.groups.EditGroupForm.systemGroupWarning' ) );
      warningMsgPanel.setVisible( true );

      me.down( '#name' ).setReadOnly( true );

      // TODO: implement
      // me.down( '#permissions' ).setReadOnly( true );//
      // me.down( '#users' ).setReadOnly( true );
    }
  },


  /**
   * Returns the {@link Group} instance with data from this form.
   */
  getGroupRecord : function ()
  {
    var me = this;

    var form = me.getForm();
    var fieldValues = form.getFieldValues();

    return QDesk.util.security.GroupUtils.createGroup( fieldValues );
  },


  /**
   * Returns the array with all stores used by this form.
   * @return the array with all stores used by this form.
   */
  getFormStores : function ()
  {
    var me = this;

    var stores = [];

    stores.push( me.down( '#permissions' ).getStore() );
    stores.push( me.down( '#users' ).getStore() );

    return stores;
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.on( 'afterrender', function ()
    {
      if ( me.qdesk.mode == QDesk.view.work.security.groups.EditGroupFormMode.ADD )
      {
        QDesk.util.FormUtils.focusFirstEditableField( me );
      }
      else
      {
        // QDesk.util.FormUtils.focusDefaultButton( me );
      }
    } );

    me.down( '#actionButton' ).on( 'click', function ()
    {
      var group = me.getGroupRecord();
      me.fireEvent( 'saveGroup', group );
    } );
  },


  /**
   * Sets the default values of individual form fields.
   */
  _setDefaultFormValues : function ()
  {
    var me = this;

    // TODO: implement
  },


  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.security.groups.EditGroupFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.security.groups.EditGroupFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  },


  _getRequiredPermissions : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.security.groups.EditGroupFormMode.ADD:
        return [ QDesk.model.security.Permission.GROUP_ADD ];

      case QDesk.view.work.security.groups.EditGroupFormMode.EDIT:
        return [ QDesk.model.security.Permission.GROUP_UPDATE ];

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
